@@ -5,10 +5,30 @@
 "use strict";
 
 const {
+  START_WORKER,
+  UNREGISTER_WORKER,
   UPDATE_CAN_DEBUG_WORKERS,
   UPDATE_WORKERS,
 } = require("devtools/client/application/src/constants");
 
+function startWorker(worker) {
+  const { registrationFront } = worker;
+  registrationFront.start();
+
+  return {
+    type: START_WORKER,
+  };
+}
+
+function unregisterWorker(registration) {
+  const { registrationFront } = registration;
+  registrationFront.unregister();
+
+  return {
+    type: UNREGISTER_WORKER,
+  };
+}
+
 function updateWorkers(workers) {
   return {
     type: UPDATE_WORKERS,
@@ -24,6 +44,8 @@ function updateCanDebugWorkers(canDebugWorkers) {
 }
 
 module.exports = {
+  startWorker,
+  unregisterWorker,
   updateCanDebugWorkers,
   updateWorkers,
 };
