@@ -2,18 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-// @flow
 import { isFulfilled } from "./async-value";
 import { findClosestFunction } from "./ast";
 import { correctIndentation } from "./indentation";
-import type { SourceWithContent } from "../types";
-import type { Symbols } from "../reducers/ast";
 
-export function findFunctionText(
-  line: number,
-  source: SourceWithContent,
-  symbols: ?Symbols
-): ?string {
+export function findFunctionText(line, source, symbols) {
   const func = findClosestFunction(symbols, {
     sourceId: source.id,
     line,
