@@ -2,39 +2,17 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-// @flow
+import { getValidatedResource, getResourceValues } from "./core";
 
-import {
-  getValidatedResource,
-  getResourceValues,
-  type ResourceState,
-  type Id,
-  type ResourceBound,
-  type ResourceIdentity,
-} from "./core";
-
-export type ResourceMap<R: ResourceBound, Mapped> = (
-  R,
-  ResourceIdentity
-) => Mapped;
-
-export function hasResource<R: ResourceBound>(
-  state: ResourceState<R>,
-  id: Id<R>
-): boolean %checks {
+export function hasResource(state, id) {
   return !!getValidatedResource(state, id);
 }
 
-export function getResourceIds<R: ResourceBound>(
-  state: ResourceState<R>
-): Array<Id<R>> {
+export function getResourceIds(state) {
   return Object.keys(getResourceValues(state));
 }
 
-export function getResource<R: ResourceBound>(
-  state: ResourceState<R>,
-  id: Id<R>
-): R {
+export function getResource(state, id) {
   const validatedState = getValidatedResource(state, id);
   if (!validatedState) {
     throw new Error(`Resource ${id} does not exist`);
@@ -42,11 +20,7 @@ export function getResource<R: ResourceBound>(
   return validatedState.values[id];
 }
 
-export function getMappedResource<R: ResourceBound, Mapped>(
-  state: ResourceState<R>,
-  id: Id<R>,
-  map: ResourceMap<R, Mapped>
-): Mapped {
+export function getMappedResource(state, id, map) {
   const validatedState = getValidatedResource(state, id);
   if (!validatedState) {
     throw new Error(`Resource ${id} does not exist`);
