@@ -1,30 +1,33 @@
-
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 /* eslint no-unused-vars: [2, {"vars": "local"}] */
+/* import-globals-from ../../../shared/test/telemetry-test-helpers.js */
 /* import-globals-from ../../test/head.js */
 "use strict";
 
 // Import the inspector's head.js first (which itself imports shared-head.js).
 Services.scriptloader.loadSubScript(
   "chrome://mochitests/content/browser/devtools/client/inspector/test/head.js",
-  this);
+  this
+);
 
-var {getInplaceEditorForSpan: inplaceEditor} = require("devtools/client/shared/inplace-editor");
+// Load the shared Redux helpers into this compartment.
+/* import-globals-from ../../../shared/test/shared-redux-head.js */
+Services.scriptloader.loadSubScript(
+  "chrome://mochitests/content/browser/devtools/client/shared/test/shared-redux-head.js",
+  this
+);
+
+var {
+  getInplaceEditorForSpan: inplaceEditor,
+} = require("devtools/client/shared/inplace-editor");
 var clipboard = require("devtools/shared/platform/clipboard");
-var {ActorRegistryFront} = require("devtools/shared/fronts/actor-registry");
 
 // If a test times out we want to see the complete log and not just the last few
 // lines.
 SimpleTest.requestCompleteLog();
 
-// Set the testing flag on DevToolsUtils and reset it when the test ends
-flags.testing = true;
-registerCleanupFunction(() => {
-  flags.testing = false;
-});
-
 // Toggle this pref on to see all DevTools event communication. This is hugely
 // useful for fixing race conditions.
 // Services.prefs.setBoolPref("devtools.dump.emit", true);
@@ -35,7 +38,6 @@ registerCleanupFunction(() => {
   Services.prefs.clearUserPref("devtools.inspector.htmlPanelOpen");
   Services.prefs.clearUserPref("devtools.inspector.sidebarOpen");
   Services.prefs.clearUserPref("devtools.markup.pagesize");
-  Services.prefs.clearUserPref("dom.webcomponents.shadowdom.enabled");
   Services.prefs.clearUserPref("devtools.inspector.showAllAnonymousContent");
 });
 
@@ -48,25 +50,12 @@ registerCleanupFunction(() => {
  * @param {String} filePath The file path, relative to the current directory.
  *                 Examples:
  *                 - "helper_attributes_test_runner.js"
- *                 - "../../../commandline/test/helpers.js"
  */
 function loadHelperScript(filePath) {
-  let testDir = gTestPath.substr(0, gTestPath.lastIndexOf("/"));
+  const testDir = gTestPath.substr(0, gTestPath.lastIndexOf("/"));
   Services.scriptloader.loadSubScript(testDir + "/" + filePath, this);
 }
 
-/**
- * Reload the current page
- * @return a promise that resolves when the inspector has emitted the event
- * new-root
- */
-function reloadPage(inspector, testActor) {
-  info("Reloading the page");
-  let newRoot = inspector.once("new-root");
-  testActor.reload();
-  return newRoot;
-}
-
 /**
  * Get the MarkupContainer object instance that corresponds to the given
  * NodeFront
@@ -75,7 +64,7 @@ function reloadPage(inspector, testActor) {
  * loaded in the toolbox
  * @return {MarkupContainer}
  */
-function getContainerForNodeFront(nodeFront, {markup}) {
+function getContainerForNodeFront(nodeFront, { markup }) {
   return markup.getContainer(nodeFront);
 }
 
@@ -88,11 +77,14 @@ function getContainerForNodeFront(nodeFront, {markup}) {
  * @param {Boolean} Set to true in the event that the node shouldn't be found.
  * @return {MarkupContainer}
  */
-var getContainerForSelector =
-Task.async(function* (selector, inspector, expectFailure = false) {
+var getContainerForSelector = async function(
+  selector,
+  inspector,
+  expectFailure = false
+) {
   info("Getting the markup-container for node " + selector);
-  let nodeFront = yield getNodeFront(selector, inspector);
-  let container = getContainerForNodeFront(nodeFront, inspector);
+  const nodeFront = await getNodeFront(selector, inspector);
+  const container = getContainerForNodeFront(nodeFront, inspector);
 
   if (expectFailure) {
     ok(!container, "Shouldn't find markup-container for selector: " + selector);
@@ -101,20 +93,22 @@ Task.async(function* (selector, inspector, expectFailure = false) {
   }
 
   return container;
-});
+};
 
 /**
  * Retrieve the nodeValue for the firstChild of a provided selector on the content page.
  *
  * @param {String} selector
- * @param {TestActorFront} testActor The current TestActorFront instance.
  * @return {String} the nodeValue of the first
  */
-function* getFirstChildNodeValue(selector, testActor) {
-  let nodeValue = yield testActor.eval(`
-    document.querySelector("${selector}").firstChild.nodeValue;
-  `);
-  return nodeValue;
+function getFirstChildNodeValue(selector) {
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [selector],
+    _selector => {
+      return content.document.querySelector(_selector).firstChild.nodeValue;
+    }
+  );
 }
 
 /**
@@ -125,7 +119,7 @@ function* getFirstChildNodeValue(selector, testActor) {
  * @return a promise that resolves when all queued children updates have been
  * handled
  */
-function waitForChildrenUpdated({markup}) {
+function waitForChildrenUpdated({ markup }) {
   info("Waiting for queued children updates to be handled");
   return new Promise(resolve => {
     markup._waitForChildren().then(() => {
@@ -142,21 +136,27 @@ function waitForChildrenUpdated({markup}) {
  * loaded in the toolbox
  * @return {Promise} Resolves when the node has been selected.
  */
-var clickContainer = Task.async(function* (selector, inspector) {
+var clickContainer = async function(selector, inspector) {
   info("Clicking on the markup-container for node " + selector);
 
-  let nodeFront = yield getNodeFront(selector, inspector);
-  let container = getContainerForNodeFront(nodeFront, inspector);
+  const nodeFront = await getNodeFront(selector, inspector);
+  const container = getContainerForNodeFront(nodeFront, inspector);
 
-  let updated = container.selected
-                ? promise.resolve()
-                : inspector.once("inspector-updated");
-  EventUtils.synthesizeMouseAtCenter(container.tagLine, {type: "mousedown"},
-    inspector.markup.doc.defaultView);
-  EventUtils.synthesizeMouseAtCenter(container.tagLine, {type: "mouseup"},
-    inspector.markup.doc.defaultView);
+  const updated = container.selected
+    ? Promise.resolve()
+    : inspector.once("inspector-updated");
+  EventUtils.synthesizeMouseAtCenter(
+    container.tagLine,
+    { type: "mousedown" },
+    inspector.markup.doc.defaultView
+  );
+  EventUtils.synthesizeMouseAtCenter(
+    container.tagLine,
+    { type: "mouseup" },
+    inspector.markup.doc.defaultView
+  );
   return updated;
-});
+};
 
 /**
  * Focus a given editable element, enter edit mode, set value, and commit
@@ -169,7 +169,7 @@ var clickContainer = Task.async(function* (selector, inspector) {
 function setEditableFieldValue(field, value, inspector) {
   field.focus();
   EventUtils.sendKey("return", inspector.panelWin);
-  let input = inplaceEditor(field).input;
+  const input = inplaceEditor(field).input;
   ok(input, "Found editable field for setting value: " + value);
   input.value = value;
   EventUtils.sendKey("return", inspector.panelWin);
@@ -185,17 +185,17 @@ function setEditableFieldValue(field, value, inspector) {
  * loaded in the toolbox
  * @return a promise that resolves when the node has mutated
  */
-var addNewAttributes = Task.async(function* (selector, text, inspector) {
+var addNewAttributes = async function(selector, text, inspector) {
   info(`Entering text "${text}" in new attribute field for node ${selector}`);
 
-  let container = yield focusNode(selector, inspector);
+  const container = await focusNode(selector, inspector);
   ok(container, "The container for '" + selector + "' was found");
 
   info("Listening for the markupmutation event");
-  let nodeMutated = inspector.once("markupmutation");
+  const nodeMutated = inspector.once("markupmutation");
   setEditableFieldValue(container.editor.newAttr, text, inspector);
-  yield nodeMutated;
-});
+  await nodeMutated;
+};
 
 /**
  * Checks that a node has the given attributes.
@@ -203,24 +203,28 @@ var addNewAttributes = Task.async(function* (selector, text, inspector) {
  * @param {String} selector The selector for the node to check.
  * @param {Object} expected An object containing the attributes to check.
  *        e.g. {id: "id1", class: "someclass"}
- * @param {TestActorFront} testActor The current TestActorFront instance.
  *
  * Note that node.getAttribute() returns attribute values provided by the HTML
  * parser. The parser only provides unescaped entities so &amp; will return &.
  */
-var assertAttributes = Task.async(function* (selector, expected, testActor) {
-  let {attributes: actual} = yield testActor.getNodeInfo(selector);
-
-  is(actual.length, Object.keys(expected).length,
-    "The node " + selector + " has the expected number of attributes.");
-  for (let attr in expected) {
-    let foundAttr = actual.find(({name}) => name === attr);
-    let foundValue = foundAttr ? foundAttr.value : undefined;
+var assertAttributes = async function(selector, expected) {
+  const actualAttributes = await getContentPageElementAttributes(selector);
+  is(
+    actualAttributes.length,
+    Object.keys(expected).length,
+    "The node " + selector + " has the expected number of attributes."
+  );
+  for (const attr in expected) {
+    const foundAttr = actualAttributes.find(({ name }) => name === attr);
+    const foundValue = foundAttr ? foundAttr.value : undefined;
     ok(foundAttr, "The node " + selector + " has the attribute " + attr);
-    is(foundValue, expected[attr],
-      "The node " + selector + " has the correct " + attr + " attribute value");
+    is(
+      foundValue,
+      expected[attr],
+      "The node " + selector + " has the correct " + attr + " attribute value"
+    );
   }
-});
+};
 
 /**
  * Undo the last markup-view action and wait for the corresponding mutation to
@@ -231,13 +235,13 @@ var assertAttributes = Task.async(function* (selector, expected, testActor) {
  * rejects if no undo action is possible
  */
 function undoChange(inspector) {
-  let canUndo = inspector.markup.undo.canUndo();
+  const canUndo = inspector.markup.undo.canUndo();
   ok(canUndo, "The last change in the markup-view can be undone");
   if (!canUndo) {
-    return promise.reject();
+    return Promise.reject();
   }
 
-  let mutated = inspector.once("markupmutation");
+  const mutated = inspector.once("markupmutation");
   inspector.markup.undo.undo();
   return mutated;
 }
@@ -251,13 +255,13 @@ function undoChange(inspector) {
  * rejects if no redo action is possible
  */
 function redoChange(inspector) {
-  let canRedo = inspector.markup.undo.canRedo();
+  const canRedo = inspector.markup.undo.canRedo();
   ok(canRedo, "The last change in the markup-view can be redone");
   if (!canRedo) {
-    return promise.reject();
+    return Promise.reject();
   }
 
-  let mutated = inspector.once("markupmutation");
+  const mutated = inspector.once("markupmutation");
   inspector.markup.undo.redo();
   return mutated;
 }
@@ -278,8 +282,8 @@ function getSelectorSearchBox(inspector) {
  * to subscribe to events and react accordingly.
  */
 function searchUsingSelectorSearch(selector, inspector) {
-  info("Entering \"" + selector + "\" into the selector-search input field");
-  let field = getSelectorSearchBox(inspector);
+  info('Entering "' + selector + '" into the selector-search input field');
+  const field = getSelectorSearchBox(inspector);
   field.focus();
   field.value = selector;
   EventUtils.sendKey("return", inspector.panelWin);
@@ -294,17 +298,24 @@ function searchUsingSelectorSearch(selector, inspector) {
  * @return A promise that resolves with a boolean indicating whether
  *         the menu items are disabled once the menu has been checked.
  */
-var isEditingMenuDisabled = Task.async(
-function* (nodeFront, inspector, assert = true) {
+var isEditingMenuDisabled = async function(
+  nodeFront,
+  inspector,
+  assert = true
+) {
   // To ensure clipboard contains something to paste.
   clipboard.copyString("<p>test</p>");
 
-  yield selectNode(nodeFront, inspector);
-  let allMenuItems = openContextMenuAndGetAllItems(inspector);
+  await selectNode(nodeFront, inspector);
+  const allMenuItems = openContextMenuAndGetAllItems(inspector);
 
-  let deleteMenuItem = allMenuItems.find(i => i.id === "node-menu-delete");
-  let editHTMLMenuItem = allMenuItems.find(i => i.id === "node-menu-edithtml");
-  let pasteHTMLMenuItem = allMenuItems.find(i => i.id === "node-menu-pasteouterhtml");
+  const deleteMenuItem = allMenuItems.find(i => i.id === "node-menu-delete");
+  const editHTMLMenuItem = allMenuItems.find(
+    i => i.id === "node-menu-edithtml"
+  );
+  const pasteHTMLMenuItem = allMenuItems.find(
+    i => i.id === "node-menu-pasteouterhtml"
+  );
 
   if (assert) {
     ok(deleteMenuItem.disabled, "Delete menu item is disabled");
@@ -312,10 +323,12 @@ function* (nodeFront, inspector, assert = true) {
     ok(pasteHTMLMenuItem.disabled, "Paste HTML menu item is disabled");
   }
 
-  return deleteMenuItem.disabled &&
-         editHTMLMenuItem.disabled &&
-         pasteHTMLMenuItem.disabled;
-});
+  return (
+    deleteMenuItem.disabled &&
+    editHTMLMenuItem.disabled &&
+    pasteHTMLMenuItem.disabled
+  );
+};
 
 /**
  * Check to see if the inspector menu items for editing are enabled.
@@ -326,17 +339,20 @@ function* (nodeFront, inspector, assert = true) {
  * @return A promise that resolves with a boolean indicating whether
  *         the menu items are enabled once the menu has been checked.
  */
-var isEditingMenuEnabled = Task.async(
-function* (nodeFront, inspector, assert = true) {
+var isEditingMenuEnabled = async function(nodeFront, inspector, assert = true) {
   // To ensure clipboard contains something to paste.
   clipboard.copyString("<p>test</p>");
 
-  yield selectNode(nodeFront, inspector);
-  let allMenuItems = openContextMenuAndGetAllItems(inspector);
+  await selectNode(nodeFront, inspector);
+  const allMenuItems = openContextMenuAndGetAllItems(inspector);
 
-  let deleteMenuItem = allMenuItems.find(i => i.id === "node-menu-delete");
-  let editHTMLMenuItem = allMenuItems.find(i => i.id === "node-menu-edithtml");
-  let pasteHTMLMenuItem = allMenuItems.find(i => i.id === "node-menu-pasteouterhtml");
+  const deleteMenuItem = allMenuItems.find(i => i.id === "node-menu-delete");
+  const editHTMLMenuItem = allMenuItems.find(
+    i => i.id === "node-menu-edithtml"
+  );
+  const pasteHTMLMenuItem = allMenuItems.find(
+    i => i.id === "node-menu-pasteouterhtml"
+  );
 
   if (assert) {
     ok(!deleteMenuItem.disabled, "Delete menu item is enabled");
@@ -344,10 +360,12 @@ function* (nodeFront, inspector, assert = true) {
     ok(!pasteHTMLMenuItem.disabled, "Paste HTML menu item is enabled");
   }
 
-  return !deleteMenuItem.disabled &&
-         !editHTMLMenuItem.disabled &&
-         !pasteHTMLMenuItem.disabled;
-});
+  return (
+    !deleteMenuItem.disabled &&
+    !editHTMLMenuItem.disabled &&
+    !pasteHTMLMenuItem.disabled
+  );
+};
 
 /**
  * Wait for all current promises to be resolved. See this as executeSoon that
@@ -359,6 +377,33 @@ function promiseNextTick() {
   });
 }
 
+/**
+ * `await` with timeout.
+ *
+ * Usage:
+ *   const badgeEventAdded = inspector.markup.once("badge-added-event");
+ *   ...
+ *   const result = await awaitWithTimeout(badgeEventAdded, 3000);
+ *   is(result, "timeout", "Ensure that no event badges were added");
+ *
+ * @param  {Promise} promise
+ *         Promise to resolve
+ * @param  {Number} ms
+ *         Milliseconds to wait.
+ * @return "timeout" on timeout, otherwise the result of the fulfilled promise.
+ */
+async function awaitWithTimeout(promise, ms) {
+  const timeout = new Promise(resolve => {
+    // eslint-disable-next-line
+    const wait = setTimeout(() => {
+      clearTimeout(wait);
+      resolve("timeout");
+    }, ms);
+  });
+
+  return Promise.race([promise, timeout]);
+}
+
 /**
  * Collapses the current text selection in an input field and tabs to the next
  * field.
@@ -376,11 +421,9 @@ function collapseSelectionAndTab(inspector) {
  */
 function collapseSelectionAndShiftTab(inspector) {
   // collapse selection and move caret to end
-  EventUtils.synthesizeKey("VK_TAB", { shiftKey: true },
-    inspector.panelWin);
+  EventUtils.synthesizeKey("VK_TAB", { shiftKey: true }, inspector.panelWin);
   // previous element
-  EventUtils.synthesizeKey("VK_TAB", { shiftKey: true },
-    inspector.panelWin);
+  EventUtils.synthesizeKey("VK_TAB", { shiftKey: true }, inspector.panelWin);
 }
 
 /**
@@ -390,10 +433,10 @@ function collapseSelectionAndShiftTab(inspector) {
  * @param {Boolean} editMode Whether or not the attribute should be in edit mode
  */
 function checkFocusedAttribute(attrName, editMode) {
-  let focusedAttr = Services.focus.focusedElement;
+  const focusedAttr = Services.focus.focusedElement;
   ok(focusedAttr, "Has a focused element");
 
-  let dataAttr = focusedAttr.parentNode.dataset.attr;
+  const dataAttr = focusedAttr.parentNode.dataset.attr;
   is(dataAttr, attrName, attrName + " attribute editor is currently focused.");
   if (editMode) {
     // Using a multiline editor for attributes, the focused element should be a textarea.
@@ -412,68 +455,13 @@ function checkFocusedAttribute(attrName, editMode) {
  *         A promise that resolves with an array of attribute names
  *         (e.g. ["id", "class", "href"])
  */
-var getAttributesFromEditor = Task.async(function* (selector, inspector) {
-  let nodeList = (yield getContainerForSelector(selector, inspector))
-    .tagLine.querySelectorAll("[data-attr]");
+var getAttributesFromEditor = async function(selector, inspector) {
+  const nodeList = (
+    await getContainerForSelector(selector, inspector)
+  ).tagLine.querySelectorAll("[data-attr]");
 
   return [...nodeList].map(node => node.getAttribute("data-attr"));
-});
-
-/**
- * Registers new backend tab actor.
- *
- * @param {DebuggerClient} client RDP client object (toolbox.target.client)
- * @param {Object} options Configuration object with the following options:
- *
- * - moduleUrl {String}: URL of the module that contains actor implementation.
- * - prefix {String}: prefix of the actor.
- * - actorClass {ActorClassWithSpec}: Constructor object for the actor.
- * - frontClass {FrontClassWithSpec}: Constructor object for the front part
- * of the registered actor.
- *
- * @returns {Promise} A promise that is resolved when the actor is registered.
- * The resolved value has two properties:
- *
- * - registrar {ActorActor}: A handle to the registered actor that allows
- * unregistration.
- * - form {Object}: The JSON actor form provided by the server.
- */
-function registerTabActor(client, options) {
-  let moduleUrl = options.moduleUrl;
-
-  return client.listTabs().then(response => {
-    let config = {
-      prefix: options.prefix,
-      constructor: options.actorClass,
-      type: { tab: true },
-    };
-
-    // Register the custom actor on the backend.
-    let registry = ActorRegistryFront(client, response);
-    return registry.registerActor(moduleUrl, config).then(registrar => {
-      return client.getTab().then(tabResponse => ({
-        registrar: registrar,
-        form: tabResponse.tab
-      }));
-    });
-  });
-}
-
-/**
- * A helper for unregistering an existing backend actor.
- *
- * @param {ActorActor} registrar A handle to the registered actor
- * that has been received after registration.
- * @param {Front} Corresponding front object.
- *
- * @returns A promise that is resolved when the unregistration
- * has finished.
- */
-function unregisterActor(registrar, front) {
-  return front.detach().then(() => {
-    return registrar.unregister();
-  });
-}
+};
 
 /**
  * Simulate dragging a MarkupContainer by calling its mousedown and mousemove
@@ -484,13 +472,19 @@ function unregisterActor(registrar, front) {
  * @param {Number} xOffset Optional x offset to drag by.
  * @param {Number} yOffset Optional y offset to drag by.
  */
-function* simulateNodeDrag(inspector, selector, xOffset = 10, yOffset = 10) {
-  let container = typeof selector === "string"
-                  ? yield getContainerForSelector(selector, inspector)
-                  : selector;
-  let rect = container.tagLine.getBoundingClientRect();
-  let scrollX = inspector.markup.doc.documentElement.scrollLeft;
-  let scrollY = inspector.markup.doc.documentElement.scrollTop;
+async function simulateNodeDrag(
+  inspector,
+  selector,
+  xOffset = 10,
+  yOffset = 10
+) {
+  const container =
+    typeof selector === "string"
+      ? await getContainerForSelector(selector, inspector)
+      : selector;
+  const rect = container.tagLine.getBoundingClientRect();
+  const scrollX = inspector.markup.doc.documentElement.scrollLeft;
+  const scrollY = inspector.markup.doc.documentElement.scrollTop;
 
   info("Simulate mouseDown on element " + selector);
   container._onMouseDown({
@@ -499,19 +493,19 @@ function* simulateNodeDrag(inspector, selector, xOffset = 10, yOffset = 10) {
     pageX: scrollX + rect.x,
     pageY: scrollY + rect.y,
     stopPropagation: () => {},
-    preventDefault: () => {}
+    preventDefault: () => {},
   });
 
   // _onMouseDown selects the node, so make sure to wait for the
   // inspector-updated event if the current selection was different.
   if (inspector.selection.nodeFront !== container.node) {
-    yield inspector.once("inspector-updated");
+    await inspector.once("inspector-updated");
   }
 
   info("Simulate mouseMove on element " + selector);
   container.onMouseMove({
     pageX: scrollX + rect.x + xOffset,
-    pageY: scrollY + rect.y + yOffset
+    pageY: scrollY + rect.y + yOffset,
   });
 }
 
@@ -522,11 +516,12 @@ function* simulateNodeDrag(inspector, selector, xOffset = 10, yOffset = 10) {
  * @param {String|MarkupContainer} selector The selector to identify the node or
  * the MarkupContainer for this node.
  */
-function* simulateNodeDrop(inspector, selector) {
+async function simulateNodeDrop(inspector, selector) {
   info("Simulate mouseUp on element " + selector);
-  let container = typeof selector === "string"
-                  ? yield getContainerForSelector(selector, inspector)
-                  : selector;
+  const container =
+    typeof selector === "string"
+      ? await getContainerForSelector(selector, inspector)
+      : selector;
   container.onMouseUp();
   inspector.markup._onMouseUp();
 }
@@ -540,22 +535,22 @@ function* simulateNodeDrop(inspector, selector) {
  * @param {Number} xOffset Optional x offset to drag by.
  * @param {Number} yOffset Optional y offset to drag by.
  */
-function* simulateNodeDragAndDrop(inspector, selector, xOffset, yOffset) {
-  yield simulateNodeDrag(inspector, selector, xOffset, yOffset);
-  yield simulateNodeDrop(inspector, selector);
+async function simulateNodeDragAndDrop(inspector, selector, xOffset, yOffset) {
+  await simulateNodeDrag(inspector, selector, xOffset, yOffset);
+  await simulateNodeDrop(inspector, selector);
 }
 
 /**
  * Waits until the element has not scrolled for 30 consecutive frames.
  */
-function* waitForScrollStop(doc) {
-  let el = doc.documentElement;
-  let win = doc.defaultView;
+async function waitForScrollStop(doc) {
+  const el = doc.documentElement;
+  const win = doc.defaultView;
   let lastScrollTop = el.scrollTop;
   let stopFrameCount = 0;
   while (stopFrameCount < 30) {
     // Wait for a frame.
-    yield new Promise(resolve => win.requestAnimationFrame(resolve));
+    await new Promise(resolve => win.requestAnimationFrame(resolve));
 
     // Check if the element has scrolled.
     if (lastScrollTop == el.scrollTop) {
@@ -586,50 +581,94 @@ function* waitForScrollStop(doc) {
  *        - {String} pseudo: optional, "before" or "after" if the element focused after
  *        deleting the node is supposed to be a before/after pseudo-element.
  */
-function* checkDeleteAndSelection(inspector, key, {selector, focusedSelector, pseudo}) {
-  info("Test deleting node " + selector + " with " + key + ", " +
-       "expecting " + focusedSelector + " to be focused");
+async function checkDeleteAndSelection(
+  inspector,
+  key,
+  { selector, focusedSelector, pseudo }
+) {
+  info(
+    "Test deleting node " +
+      selector +
+      " with " +
+      key +
+      ", " +
+      "expecting " +
+      focusedSelector +
+      " to be focused"
+  );
 
   info("Select node " + selector + " and make sure it is focused");
-  yield selectNode(selector, inspector);
-  yield clickContainer(selector, inspector);
+  await selectNode(selector, inspector);
+  await clickContainer(selector, inspector);
 
   info("Delete the node with: " + key);
-  let mutated = inspector.once("markupmutation");
+  const mutated = inspector.once("markupmutation");
   EventUtils.sendKey(key, inspector.panelWin);
-  yield Promise.all([mutated, inspector.once("inspector-updated")]);
+  await Promise.all([mutated, inspector.once("inspector-updated")]);
 
-  let nodeFront = yield getNodeFront(focusedSelector, inspector);
+  let nodeFront = await getNodeFront(focusedSelector, inspector);
   if (pseudo) {
     // Update the selector for logging in case of failure.
     focusedSelector = focusedSelector + "::" + pseudo;
     // Retrieve the :before or :after pseudo element of the nodeFront.
-    let {nodes} = yield inspector.walker.children(nodeFront);
+    const { nodes } = await inspector.walker.children(nodeFront);
     nodeFront = pseudo === "before" ? nodes[0] : nodes[nodes.length - 1];
   }
 
-  is(inspector.selection.nodeFront, nodeFront,
-     focusedSelector + " is selected after deletion");
+  is(
+    inspector.selection.nodeFront,
+    nodeFront,
+    focusedSelector + " is selected after deletion"
+  );
 
   info("Check that the node was really removed");
-  let node = yield getNodeFront(selector, inspector);
+  let node = await getNodeFront(selector, inspector);
   ok(!node, "The node can't be found in the page anymore");
 
   info("Undo the deletion to restore the original markup");
-  yield undoChange(inspector);
-  node = yield getNodeFront(selector, inspector);
+  await undoChange(inspector);
+  node = await getNodeFront(selector, inspector);
   ok(node, "The node is back");
 }
 
 /**
- * Expand the provided markup container by clicking on the expand arrow and waiting for
- * inspector and children to update.
+ * Click on the reveal link the provided slotted container.
+ * Will resolve when selection emits "new-node-front".
  */
-async function expandContainer(inspector, container) {
-  let onChildren = waitForChildrenUpdated(inspector);
-  let onUpdated = inspector.once("inspector-updated");
-  EventUtils.synthesizeMouseAtCenter(container.expander, {},
-    inspector.markup.doc.defaultView);
-  await onChildren;
-  await onUpdated;
+async function clickOnRevealLink(inspector, container) {
+  const onSelection = inspector.selection.once("new-node-front");
+  const revealLink = container.elt.querySelector(".reveal-link");
+  const tagline = revealLink.closest(".tag-line");
+  const win = inspector.markup.doc.defaultView;
+
+  // First send a mouseover on the tagline to force the link to be displayed.
+  EventUtils.synthesizeMouseAtCenter(tagline, { type: "mouseover" }, win);
+  EventUtils.synthesizeMouseAtCenter(revealLink, {}, win);
+
+  await onSelection;
+}
+
+/**
+ * Hit `key` on the reveal link in the provided slotted container.
+ * Will resolve when selection emits "new-node-front".
+ */
+async function keydownOnRevealLink(key, inspector, container) {
+  const revealLink = container.elt.querySelector(".reveal-link");
+  const win = inspector.markup.doc.defaultView;
+
+  const root = inspector.markup.getContainer(inspector.markup._rootNode);
+  root.elt.focus();
+
+  // we need to go through a ENTER + TAB  key sequence to focus on
+  // the .reveal-link element with the keyboard
+  const revealFocused = once(revealLink, "focus");
+  EventUtils.synthesizeKey("KEY_Enter", {}, win);
+  EventUtils.synthesizeKey("KEY_Tab", {}, win);
+  info("Waiting for .reveal-link to be focused");
+  await revealFocused;
+
+  // hit `key` on the .reveal-link
+  const onSelection = inspector.selection.once("new-node-front");
+  EventUtils.synthesizeKey(key, {}, win);
+  await onSelection;
 }
