@@ -29,17 +29,27 @@ const ELEMENTS = [
 ];
 
 add_task(async function() {
-  const { inspector, testActor } = await openInspectorForURL(TEST_URL);
+  const { inspector, highlighterTestFront } = await openInspectorForURL(
+    TEST_URL
+  );
 
   info("Show the box-model highlighter");
   const divFront = await getNodeFront("div", inspector);
-  await inspector.highlighter.showBoxModel(divFront);
+  await inspector.highlighters.showHighlighterTypeForNode(
+    inspector.highlighters.TYPES.BOXMODEL,
+    divFront
+  );
 
   for (const id of ELEMENTS) {
-    const foundId = await testActor.getHighlighterNodeAttribute(id, "id");
+    const foundId = await highlighterTestFront.getHighlighterNodeAttribute(
+      id,
+      "id"
+    );
     is(foundId, id, "Element " + id + " found");
   }
 
   info("Hide the box-model highlighter");
-  await inspector.highlighter.hideBoxModel();
+  await inspector.highlighters.hideHighlighterType(
+    inspector.highlighters.TYPES.BOXMODEL
+  );
 });
