@@ -63,8 +63,6 @@ async function addJsonViewTab(
   const tab = await Promise.race([tabAdded, tabLoaded]);
   const browser = tab.linkedBrowser;
 
-  // Load devtools/shared/test/frame-script-utils.js
-  loadFrameScriptUtils();
   const rootDir = getRootDirectory(gTestPath);
 
   // Catch RequireJS errors (usually timeouts)
@@ -94,10 +92,6 @@ async function addJsonViewTab(
         throw new Error("The JSON Viewer did not load.");
       }
 
-      // Load frame script with helpers for JSON View tests.
-      const frameScriptUrl = data.rootDir + "doc_frame_script.js";
-      Services.scriptloader.loadSubScript(frameScriptUrl, {}, "UTF-8");
-
       const docReadyStates = ["loading", "interactive", "complete"];
       const docReadyIndex = docReadyStates.indexOf(data.docReadyState);
       const appReadyStates = ["uninitialized", ...docReadyStates];
@@ -152,21 +146,27 @@ function clickJsonNode(selector) {
 function selectJsonViewContentTab(name) {
   info("Selecting tab: '" + name + "'");
 
-  const browser = gBrowser.selectedBrowser;
-  const selector = ".tabs-menu .tabs-menu-item." + name + " a";
-  return BrowserTestUtils.synthesizeMouseAtCenter(selector, {}, browser);
+  // eslint-disable-next-line no-shadow
+  return ContentTask.spawn(gBrowser.selectedBrowser, name, async name => {
+    const selector = ".tabs-menu .tabs-menu-item." + CSS.escape(name) + " a";
+    const element = content.document.querySelector(selector);
+    is(element.getAttribute("aria-selected"), "false", "Tab not selected yet");
+    await new Promise(resolve => {
+      content.addEventListener("TabChanged", resolve, { once: true });
+      element.click();
+    });
+    is(element.getAttribute("aria-selected"), "true", "Tab is now selected");
+  });
 }
 
 function getElementCount(selector) {
   info("Get element count: '" + selector + "'");
 
-  const data = {
-    selector: selector,
-  };
-
-  return executeInContent("Test:JsonView:GetElementCount", data).then(
-    result => {
-      return result.count;
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [selector],
+    selectorChild => {
+      return content.document.querySelectorAll(selectorChild).length;
     }
   );
 }
@@ -174,32 +174,42 @@ function getElementCount(selector) {
 function getElementText(selector) {
   info("Get element text: '" + selector + "'");
 
-  const data = {
-    selector: selector,
-  };
-
-  return executeInContent("Test:JsonView:GetElementText", data).then(result => {
-    return result.text;
-  });
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [selector],
+    selectorChild => {
+      const element = content.document.querySelector(selectorChild);
+      return element ? element.textContent : null;
+    }
+  );
 }
 
 function getElementAttr(selector, attr) {
   info("Get attribute '" + attr + "' for element '" + selector + "'");
 
-  const data = { selector, attr };
-  return executeInContent("Test:JsonView:GetElementAttr", data).then(
-    result => result.text
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [selector, attr],
+    (selectorChild, attrChild) => {
+      const element = content.document.querySelector(selectorChild);
+      return element ? element.getAttribute(attrChild) : null;
+    }
   );
 }
 
 function focusElement(selector) {
   info("Focus element: '" + selector + "'");
 
-  const data = {
-    selector: selector,
-  };
-
-  return executeInContent("Test:JsonView:FocusElement", data);
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [selector],
+    selectorChild => {
+      const element = content.document.querySelector(selectorChild);
+      if (element) {
+        element.focus();
+      }
+    }
+  );
 }
 
 /**
@@ -211,12 +221,20 @@ function focusElement(selector) {
 function sendString(str, selector) {
   info("Send string: '" + str + "'");
 
-  const data = {
-    selector: selector,
-    str: str,
-  };
+  return SpecialPowers.spawn(
+    gBrowser.selectedBrowser,
+    [selector, str],
+    (selectorChild, strChild) => {
+      if (selectorChild) {
+        const element = content.document.querySelector(selectorChild);
+        if (element) {
+          element.focus();
+        }
+      }
 
-  return executeInContent("Test:JsonView:SendString", data);
+      EventUtils.sendString(strChild, content);
+    }
+  );
 }
 
 function waitForTime(delay) {
@@ -224,7 +242,35 @@ function waitForTime(delay) {
 }
 
 function waitForFilter() {
-  return executeInContent("Test:JsonView:WaitForFilter");
+  return SpecialPowers.spawn(gBrowser.selectedBrowser, [], () => {
+    return new Promise(resolve => {
+      const firstRow = content.document.querySelector(
+        ".jsonPanelBox .treeTable .treeRow"
+      );
+
+      // Check if the filter is already set.
+      if (firstRow.classList.contains("hidden")) {
+        resolve();
+        return;
+      }
+
+      // Wait till the first row has 'hidden' class set.
+      const observer = new content.MutationObserver(function(mutations) {
+        for (let i = 0; i < mutations.length; i++) {
+          const mutation = mutations[i];
+          if (mutation.attributeName == "class") {
+            if (firstRow.classList.contains("hidden")) {
+              observer.disconnect();
+              resolve();
+              break;
+            }
+          }
+        }
+      });
+
+      observer.observe(firstRow, { attributes: true });
+    });
+  });
 }
 
 function normalizeNewLines(value) {
