@@ -130,16 +130,16 @@ const JsCallTreeView = extend(DetailsSubview, {
     const optimizations = JITOptimizationsView({
       frameData,
       optimizationSites,
-      onViewSourceInDebugger: (url, line, column) => {
-        PerformanceController.toolbox
-          .viewSourceInDebugger(url, line, column)
-          .then(success => {
+      onViewSourceInDebugger: ({ url, line, column }) => {
+        PerformanceController.viewSourceInDebugger(url, line, column).then(
+          success => {
             if (success) {
               this.emit(EVENTS.SOURCE_SHOWN_IN_JS_DEBUGGER);
             } else {
               this.emit(EVENTS.SOURCE_NOT_FOUND_IN_JS_DEBUGGER);
             }
-          });
+          }
+        );
       },
     });
 
@@ -153,15 +153,15 @@ const JsCallTreeView = extend(DetailsSubview, {
    */
   _onLink: function(treeItem) {
     const { url, line, column } = treeItem.frame.getInfo();
-    PerformanceController.toolbox
-      .viewSourceInDebugger(url, line, column)
-      .then(success => {
+    PerformanceController.viewSourceInDebugger(url, line, column).then(
+      success => {
         if (success) {
           this.emit(EVENTS.SOURCE_SHOWN_IN_JS_DEBUGGER);
         } else {
           this.emit(EVENTS.SOURCE_NOT_FOUND_IN_JS_DEBUGGER);
         }
-      });
+      }
+    );
   },
 
   /**
