@@ -135,8 +135,6 @@ var closeRDM = async function(tab, options) {
  * removes the tab. The final argument is an options object, with these
  * optional properties:
  *
- * usingBrowserUI: the devtools.responsive.browserUI.enabled pref is set
- *   to the truthiness of this value (default false).
  * onlyPrefAndTask: if truthy, only the pref will be set and the task
  *   will be called, with none of the tab creation/teardown or open/close
  *   of RDM (default false).
@@ -147,36 +145,27 @@ var closeRDM = async function(tab, options) {
  *
  *   addRDMTaskWithPreAndPost(
  *     TEST_URL,
- *     async function preTask({ message, browser, usingBrowserUI }) {
+ *     async function preTask({ message, browser }) {
  *       // Your pre-task goes here...
  *     },
- *     async function task({ ui, manager, message, browser, usingBrowserUI,
- *                           preTaskValue }) {
+ *     async function task({ ui, manager, message, browser, preTaskValue, tab }) {
  *       // Your task goes here...
  *     },
- *     async function postTask({ message, browser, usingBrowserUI,
- *                               preTaskValue, taskValue }) {
+ *     async function postTask({ message, browser, preTaskValue, taskValue }) {
  *       // Your post-task goes here...
  *     },
- *     { usingBrowserUI: true, waitForDeviceList: true }
+ *     { waitForDeviceList: true }
  *   );
  */
 function addRDMTaskWithPreAndPost(url, preTask, task, postTask, options) {
-  // Interpret our options.
-  let usingBrowserUI = false;
   let onlyPrefAndTask = false;
   let waitForDeviceList = false;
   if (typeof options == "object") {
-    usingBrowserUI = !!options.usingBrowserUI;
     onlyPrefAndTask = !!options.onlyPrefAndTask;
     waitForDeviceList = !!options.waitForDeviceList;
   }
 
   add_task(async function() {
-    await SpecialPowers.pushPrefEnv({
-      set: [["devtools.responsive.browserUI.enabled", usingBrowserUI]],
-    });
-
     let tab;
     let browser;
     let preTaskValue = null;
@@ -189,27 +178,14 @@ function addRDMTaskWithPreAndPost(url, preTask, task, postTask, options) {
       browser = tab.linkedBrowser;
 
       if (preTask) {
-        preTaskValue = await preTask({ message, browser, usingBrowserUI });
+        preTaskValue = await preTask({ message, browser });
       }
 
       const rdmValues = await openRDM(tab);
       ui = rdmValues.ui;
       manager = rdmValues.manager;
 
-      // Always wait for the post-init message.
-      await message.wait(ui.toolWindow, "post-init");
-
-      // Always wait for the viewport to be added.
-      const { store } = ui.toolWindow;
-      await waitUntilState(store, state => state.viewports.length == 1);
-
-      if (waitForDeviceList) {
-        // Wait until the device list has been loaded.
-        await waitUntilState(
-          store,
-          state => state.devices.listState == localTypes.loadableState.LOADED
-        );
-      }
+      await waitForRDMLoaded(ui, { waitForDeviceList });
     }
 
     try {
@@ -218,17 +194,11 @@ function addRDMTaskWithPreAndPost(url, preTask, task, postTask, options) {
         manager,
         message,
         browser,
-        usingBrowserUI,
         preTaskValue,
+        tab,
       });
     } catch (err) {
-      ok(
-        false,
-        "Got an error with usingBrowserUI " +
-          usingBrowserUI +
-          ": " +
-          DevToolsUtils.safeErrorString(err)
-      );
+      ok(false, "Got an error: " + DevToolsUtils.safeErrorString(err));
     }
 
     if (!onlyPrefAndTask) {
@@ -237,7 +207,6 @@ function addRDMTaskWithPreAndPost(url, preTask, task, postTask, options) {
         await postTask({
           message,
           browser,
-          usingBrowserUI,
           preTaskValue,
           taskValue,
         });
@@ -251,21 +220,39 @@ function addRDMTaskWithPreAndPost(url, preTask, task, postTask, options) {
   });
 }
 
+/**
+ * Wait for the RDM UI to be fully loaded
+ */
+async function waitForRDMLoaded(ui, { waitForDeviceList }) {
+  // Always wait for the post-init message.
+  await message.wait(ui.toolWindow, "post-init");
+
+  // Always wait for the viewport to be added.
+  const { store } = ui.toolWindow;
+  await waitUntilState(store, state => state.viewports.length == 1);
+
+  if (waitForDeviceList) {
+    // Wait until the device list has been loaded.
+    await waitUntilState(
+      store,
+      state => state.devices.listState == localTypes.loadableState.LOADED
+    );
+  }
+}
+
 /**
  * This is a simplified version of addRDMTaskWithPreAndPost. Adds a new test
  * task that adds a tab with the given URL, opens responsive design mode,
- * closes responsive design mode, and removes the tab. If
- * includeBrowserEmbeddedUI is truthy, the sequence will be repeated with the
- * devtools.responsive.browserUI.enabled pref set.
+ * closes responsive design mode, and removes the tab.
  *
  * Example usage:
  *
  *   addRDMTask(
  *     TEST_URL,
- *     async function task({ ui, manager, message, browser, usingBrowserUI }) {
+ *     async function task({ ui, manager, message, browser }) {
  *       // Your task goes here...
  *     },
- *     { usingBrowserUI: true, waitForDeviceList: true }
+ *     { waitForDeviceList: true }
  *   );
  */
 function addRDMTask(rdmURL, rdmTask, options) {
@@ -381,22 +368,12 @@ function dragElementBy(selector, x, y, ui) {
   };
   const endPoint = [startPoint.clientX + x, startPoint.clientY + y];
 
-  const elem = browserWindow.document.querySelector(selector);
-
-  if (!Services.prefs.getBoolPref("devtools.responsive.browserUI.enabled")) {
-    const { Simulate } = ui.toolWindow.require(
-      "devtools/client/shared/vendor/react-dom-test-utils"
-    );
-    // mousedown is a React listener, need to use its testing tools to avoid races
-    Simulate.mouseDown(elem, startPoint);
-  } else {
-    EventUtils.synthesizeMouseAtPoint(
-      startPoint.clientX,
-      startPoint.clientY,
-      { type: "mousedown" },
-      browserWindow
-    );
-  }
+  EventUtils.synthesizeMouseAtPoint(
+    startPoint.clientX,
+    startPoint.clientY,
+    { type: "mousedown" },
+    browserWindow
+  );
 
   // mousemove and mouseup are regular DOM listeners
   EventUtils.synthesizeMouseAtPoint(
@@ -413,21 +390,8 @@ function dragElementBy(selector, x, y, ui) {
   return rect;
 }
 
-async function testViewportResize(
-  ui,
-  selector,
-  moveBy,
-  expectedViewportSize,
-  expectedHandleMove
-) {
-  let resized;
-
-  if (!Services.prefs.getBoolPref("devtools.responsive.browserUI.enabled")) {
-    resized = waitForViewportResizeTo(ui, ...expectedViewportSize);
-  } else {
-    resized = ui.once("viewport-resize-dragend");
-  }
-
+async function testViewportResize(ui, selector, moveBy, expectedHandleMove) {
+  const resized = ui.once("viewport-resize-dragend");
   const startRect = dragElementBy(selector, ...moveBy, ui);
   await resized;
 
@@ -555,6 +519,26 @@ const selectNetworkThrottling = (ui, value) =>
   ]);
 
 function getSessionHistory(browser) {
+  if (Services.appinfo.sessionHistoryInParent) {
+    const browsingContext = browser.browsingContext;
+    const uri = browsingContext.currentWindowGlobal.documentURI.displaySpec;
+    const history = browsingContext.sessionHistory;
+    const body = ContentTask.spawn(browser, browsingContext, function(
+      // eslint-disable-next-line no-shadow
+      browsingContext
+    ) {
+      const docShell = browsingContext.docShell.QueryInterface(
+        Ci.nsIWebNavigation
+      );
+      return docShell.document.body;
+    });
+    /* eslint-disable no-undef */
+    const { SessionHistory } = ChromeUtils.import(
+      "resource://gre/modules/sessionstore/SessionHistory.jsm"
+    );
+    return SessionHistory.collectFromParent(uri, body, history);
+    /* eslint-enable no-undef */
+  }
   return ContentTask.spawn(browser, null, function() {
     /* eslint-disable no-undef */
     const { SessionHistory } = ChromeUtils.import(
@@ -612,7 +596,7 @@ function waitForViewportScroll(ui) {
 
 async function load(browser, url) {
   const loaded = BrowserTestUtils.browserLoaded(browser, false, null, false);
-  await BrowserTestUtils.loadURI(browser, url);
+  BrowserTestUtils.loadURI(browser, url);
   await loaded;
 }
 
@@ -643,7 +627,7 @@ function addDeviceForTest(device) {
 
 async function waitForClientClose(ui) {
   info("Waiting for RDM devtools client to close");
-  await ui.client.once("closed");
+  await ui.commands.client.once("closed");
   info("RDM's devtools client is now closed");
 }
 
@@ -656,9 +640,10 @@ async function testTouchEventsOverride(ui, expected) {
   const { document } = ui.toolWindow;
   const touchButton = document.getElementById("touch-simulation-button");
 
-  const flag = await ui.responsiveFront.getTouchEventsOverride();
+  const flag = gBrowser.selectedBrowser.browsingContext.touchEventsOverride;
+
   is(
-    flag === Ci.nsIDocShell.TOUCHEVENTS_OVERRIDE_ENABLED,
+    flag === "enabled",
     expected,
     `Touch events override should be ${expected ? "enabled" : "disabled"}`
   );
@@ -867,16 +852,13 @@ function rotateViewport(ui) {
 
 // Call this to switch between on/off support for meta viewports.
 async function setTouchAndMetaViewportSupport(ui, value) {
-  const reloadNeeded = await ui.updateTouchSimulation(value);
-  if (reloadNeeded) {
-    info("Reload is needed -- waiting for it.");
-    const reload = waitForViewportLoad(ui);
-    const browser = ui.getViewportBrowser();
-    browser.reload();
-    await reload;
-    await promiseContentReflow(ui);
-  }
-  return reloadNeeded;
+  await ui.updateTouchSimulation(value);
+  info("Reload so the new configuration applies cleanly to the page");
+  const reload = waitForViewportLoad(ui);
+  const browser = ui.getViewportBrowser();
+  browser.reload();
+  await reload;
+  await promiseContentReflow(ui);
 }
 
 // This function checks that zoom, layout viewport width and height
@@ -949,6 +931,63 @@ async function waitForDeviceAndViewportState(ui) {
   );
 }
 
+/**
+ * Wait for the content page to be rendered with the expected pixel ratio.
+ *
+ * @param {ResponsiveUI} ui
+ *        The ResponsiveUI instance.
+ * @param {Integer} expected
+ *        The expected dpr for the content page.
+ * @param {Object} options
+ * @param {Boolean} options.waitForTargetConfiguration
+ *        If set to true, the function will wait for the targetConfigurationCommand configuration
+ *        to reflect the ratio that was set. This can be used to prevent pending requests
+ *        to the actor.
+ */
+async function waitForDevicePixelRatio(
+  ui,
+  expected,
+  { waitForTargetConfiguration } = {}
+) {
+  const dpx = await SpecialPowers.spawn(
+    ui.getViewportBrowser(),
+    [{ expected }],
+    function(args) {
+      const initial = content.devicePixelRatio;
+      info(
+        `Listening for pixel ratio change ` +
+          `(current: ${initial}, expected: ${args.expected})`
+      );
+      return new Promise(resolve => {
+        const mql = content.matchMedia(`(resolution: ${args.expected}dppx)`);
+        if (mql.matches) {
+          info(`Ratio already changed to ${args.expected}dppx`);
+          resolve(content.devicePixelRatio);
+          return;
+        }
+        mql.addListener(function listener() {
+          info(`Ratio changed to ${args.expected}dppx`);
+          mql.removeListener(listener);
+          resolve(content.devicePixelRatio);
+        });
+      });
+    }
+  );
+
+  if (waitForTargetConfiguration) {
+    // Ensure the configuration was updated so we limit the risk of the client closing before
+    // the server sent back the result of the updateConfiguration call.
+    await waitFor(() => {
+      return (
+        ui.commands.targetConfigurationCommand.configuration.overrideDPPX ===
+        expected
+      );
+    });
+  }
+
+  return dpx;
+}
+
 /**
  * Navigate the selected tab to a new URL and wait for the RDM UI to switch to a new
  * target. Until Bug 1627847 is fixed, this helper should only be called when we are
