@@ -514,24 +514,25 @@ HTMLTooltip.prototype = {
     this._focusedElement = this.doc.activeElement;
 
     if (this.doc.defaultView) {
-      if (this.attachEventsTimer) {
-        this.doc.defaultView.clearTimeout(this.attachEventsTimer);
+      if (!this._pendingEventListenerPromise) {
+        // On Windows and Linux, if the tooltip is shown on mousedown/click (which is the
+        // case for the MenuButton component for example), attaching the events listeners
+        // on the window right away would trigger the callbacks; which means the tooltip
+        // would be instantly hidden. To prevent such thing, the event listeners are set
+        // on the next tick.
+        this._pendingEventListenerPromise = new Promise(resolve => {
+          this.doc.defaultView.setTimeout(() => {
+            // Update the top window reference each time in case the host changes.
+            this.topWindow = this._getTopWindow();
+            this.topWindow.addEventListener("click", this._onClick, true);
+            this.topWindow.addEventListener("mouseup", this._onMouseup, true);
+            resolve();
+          }, 0);
+        });
       }
 
-      // On Windows and Linux, if the tooltip is shown on mousedown/click (which is the
-      // case for the MenuButton component for example), attaching the events listeners
-      // on the window right away would trigger the callbacks; which means the tooltip
-      // would be instantly hidden. To prevent such thing, the event listeners are set
-      // on the next tick.
-      await new Promise(resolve => {
-        this.attachEventsTimer = this.doc.defaultView.setTimeout(() => {
-          // Update the top window reference each time in case the host changes.
-          this.topWindow = this._getTopWindow();
-          this.topWindow.addEventListener("click", this._onClick, true);
-          this.topWindow.addEventListener("mouseup", this._onMouseup, true);
-          resolve();
-        }, 0);
-      });
+      await this._pendingEventListenerPromise;
+      this._pendingEventListenerPromise = null;
     }
 
     this.emit("shown");
@@ -804,17 +805,17 @@ HTMLTooltip.prototype = {
       return;
     }
 
-    if (this.doc && this.doc.defaultView) {
-      this.doc.defaultView.clearTimeout(this.attachEventsTimer);
-    }
-
     // If the tooltip is hidden from a mouseup event, wait for a potential click event
     // to be consumed before removing event listeners.
     if (fromMouseup) {
       await new Promise(resolve => this.topWindow.setTimeout(resolve, 0));
     }
 
-    this.removeEventListeners();
+    if (this._pendingEventListenerPromise) {
+      this._pendingEventListenerPromise.then(() => this.removeEventListeners());
+    } else {
+      this.removeEventListeners();
+    }
 
     this.container.classList.remove("tooltip-visible");
     if (this.useXulWrapper) {
@@ -1009,6 +1010,9 @@ HTMLTooltip.prototype = {
     panel.setAttribute("level", "top");
     panel.setAttribute("class", "tooltip-xul-wrapper");
 
+    // Stop this appearing as an alert to accessibility.
+    panel.setAttribute("role", "presentation");
+
     return panel;
   },
 
