@@ -23,7 +23,7 @@ const { require, loader } = ChromeUtils.import(
 const { worker } = ChromeUtils.import(
   "resource://devtools/shared/worker/loader.js"
 );
-const defer = require("devtools/shared/defer");
+
 const { NetUtil } = require("resource://gre/modules/NetUtil.jsm");
 
 const Services = require("Services");
@@ -45,7 +45,7 @@ const { DevToolsServer: WorkerDevToolsServer } = worker.require(
 const { DevToolsClient } = require("devtools/client/devtools-client");
 const { ObjectFront } = require("devtools/client/fronts/object");
 const { LongStringFront } = require("devtools/client/fronts/string");
-const { TargetFactory } = require("devtools/client/framework/target");
+const { createCommandsDictionary } = require("devtools/shared/commands/index");
 
 const { addDebuggerToGlobal } = ChromeUtils.import(
   "resource://gre/modules/jsdebugger.jsm"
@@ -75,8 +75,6 @@ async function startupAddonsManager() {
   const profileDir = do_get_profile().clone();
   profileDir.append("extensions");
 
-  /* global globalThis */
-  /* See Bug 1595810 to add globalThis to eslint */
   AddonTestUtils.init(globalThis);
   AddonTestUtils.overrideCertDB();
   AddonTestUtils.appInfo = getAppInfo();
@@ -363,7 +361,7 @@ function addTestGlobal(name, server = DevToolsServer) {
 }
 
 // List the DevToolsClient |client|'s tabs, look for one whose title is
-// |title|, and apply |callback| to the packet's entry for that tab.
+// |title|.
 async function getTestTab(client, title) {
   const tabs = await client.mainRoot.listTabs();
   for (const tab of tabs) {
@@ -373,44 +371,38 @@ async function getTestTab(client, title) {
   }
   return null;
 }
-
-// Attach to |client|'s tab whose title is |title|; and return the targetFront instance
-// referring to that tab.
+/**
+ *  Attach to the client's tab whose title is specified
+ * @param {Object} client
+ * @param {Object} title
+ * @returns commands
+ */
 async function attachTestTab(client, title) {
   const descriptorFront = await getTestTab(client, title);
-  const targetFront = await descriptorFront.getTarget();
-  await targetFront.attach();
-  return targetFront;
+  const commands = await createCommandsDictionary(descriptorFront);
+  await commands.targetCommand.startListening();
+  return commands;
 }
 
-// Attach to |client|'s tab whose title is |title|, and then attach to
-// that tab's thread. Pass |callback| the thread attach response packet, a
-// TargetFront referring to the tab, and a ThreadFront referring to the
-// thread.
-async function attachTestThread(client, title, callback = () => {}) {
-  const targetFront = await attachTestTab(client, title);
+/**
+ * Attach to the client's tab whose title is specified, and then attach to
+ * that tab's thread.
+ * @param {Object} client
+ * @param {Object} title
+ * @returns {Object}
+ *         targetFront
+ *         threadFront
+ *         commands
+ */
+async function attachTestThread(client, title) {
+  const commands = await attachTestTab(client, title);
+  const targetFront = commands.targetCommand.targetFront;
   const threadFront = await targetFront.getFront("thread");
-  const onPaused = threadFront.once("paused");
   await targetFront.attachThread({
     autoBlackBox: true,
   });
-  const response = await onPaused;
-  Assert.equal(threadFront.state, "paused", "Thread client is paused");
-  Assert.ok("why" in response);
-  Assert.equal(response.why.type, "attached");
-  callback(response, targetFront, threadFront);
-  return { targetFront, threadFront };
-}
-
-// Attach to |client|'s tab whose title is |title|, attach to the tab's
-// thread, and then resume it. Pass |callback| the thread's response to
-// the 'resume' packet, a TargetFront for the tab, and a ThreadFront for the
-// thread.
-async function attachTestTabAndResume(client, title, callback = () => {}) {
-  const { targetFront, threadFront } = await attachTestThread(client, title);
-  const response = await threadFront.resume();
-  callback(response, targetFront, threadFront);
-  return { targetFront, threadFront };
+  Assert.equal(threadFront.state, "attached", "Thread front is attached");
+  return { targetFront, threadFront, commands };
 }
 
 /**
@@ -613,9 +605,10 @@ function interrupt(threadFront) {
  * @param ThreadFront threadFront
  * @returns Promise
  */
-function resumeAndWaitForPause(threadFront) {
+async function resumeAndWaitForPause(threadFront) {
   const paused = waitForPause(threadFront);
-  return resume(threadFront).then(() => paused);
+  await resume(threadFront);
+  return paused;
 }
 
 /**
@@ -658,6 +651,20 @@ async function stepOut(threadFront, frameActor) {
   return waitForPause(threadFront);
 }
 
+/**
+ * Restart specific frame and wait for the pause after the restart
+ * has been taken.
+ *
+ * @param DevToolsClient client
+ * @param ThreadFront threadFront
+ * @returns Promise
+ */
+async function restartFrame(threadFront, frameActor) {
+  dumpn("Restarting frame.");
+  await threadFront.restart(frameActor);
+  return waitForPause(threadFront);
+}
+
 /**
  * Get the list of `count` frames currently on stack, starting at the index
  * `first` for the specified thread.
@@ -740,6 +747,15 @@ async function getSourceFormById(threadFront, id) {
   return sources.find(source => source.actor == id);
 }
 
+async function checkFramesLength(threadFront, expectedFrames) {
+  const frameResponse = await threadFront.getFrames(0, null);
+  Assert.equal(
+    frameResponse.frames.length,
+    expectedFrames,
+    "Thread front has the expected number of frames"
+  );
+}
+
 /**
  * Do a reload which clears the thread debugger
  *
@@ -800,7 +816,6 @@ async function setupTestFromUrl(url) {
   await targetFront.attach();
 
   const threadFront = await attachThread(targetFront);
-  await resume(threadFront);
 
   const sourceUrl = getFileUrl(url);
   const promise = waitForNewSource(threadFront, sourceUrl);
@@ -864,7 +879,7 @@ function threadFrontTest(test, options = {}) {
 
     // Attach to the fake tab target and retrieve the ThreadFront instance.
     // Automatically resume as the thread is paused by default after attach.
-    const { targetFront, threadFront } = await attachTestTabAndResume(
+    const { targetFront, threadFront, commands } = await attachTestThread(
       client,
       scriptName
     );
@@ -885,6 +900,7 @@ function threadFrontTest(test, options = {}) {
       client,
       server,
       targetFront,
+      commands,
     };
     if (waitForFinish) {
       // Use dispatchToMainThread so that the test function does not have to
