@@ -8,12 +8,15 @@
 
 #include <algorithm>
 
-#include "mozilla/Atomics.h"
 #include "base/compiler_specific.h"
 #include "base/logging.h"
 #include "base/message_pump_default.h"
 #include "base/string_util.h"
 #include "base/thread_local.h"
+#include "GeckoProfiler.h"
+#include "mozilla/Atomics.h"
+#include "mozilla/Mutex.h"
+#include "nsThreadUtils.h"
 
 #if defined(OS_MACOSX)
 #  include "base/message_pump_mac.h"
@@ -30,12 +33,9 @@
 #  include "base/message_pump_android.h"
 #endif
 #include "nsISerialEventTarget.h"
-#ifdef MOZ_TASK_TRACER
-#  include "GeckoTaskTracer.h"
-#  include "TracedTaskCommon.h"
-#endif
 
-#include "MessagePump.h"
+#include "mozilla/ipc/MessagePump.h"
+#include "nsThreadUtils.h"
 
 using base::Time;
 using base::TimeDelta;
@@ -90,7 +90,8 @@ class MessageLoop::EventTarget : public nsISerialEventTarget,
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSIEVENTTARGET_FULL
 
-  explicit EventTarget(MessageLoop* aLoop) : mLoop(aLoop) {
+  explicit EventTarget(MessageLoop* aLoop)
+      : mMutex("MessageLoop::EventTarget"), mLoop(aLoop) {
     aLoop->AddDestructionObserver(this);
   }
 
@@ -102,10 +103,15 @@ class MessageLoop::EventTarget : public nsISerialEventTarget,
   }
 
   void WillDestroyCurrentMessageLoop() override {
+    mozilla::MutexAutoLock lock(mMutex);
+    // The MessageLoop is being destroyed and we are called from its destructor
+    // There's no real need to remove ourselves from the destruction observer
+    // list. But it makes things look tidier.
     mLoop->RemoveDestructionObserver(this);
     mLoop = nullptr;
   }
 
+  mozilla::Mutex mMutex;
   MessageLoop* mLoop;
 };
 
@@ -114,6 +120,7 @@ NS_IMPL_ISUPPORTS(MessageLoop::EventTarget, nsIEventTarget,
 
 NS_IMETHODIMP_(bool)
 MessageLoop::EventTarget::IsOnCurrentThreadInfallible() {
+  mozilla::MutexAutoLock lock(mMutex);
   return mLoop == MessageLoop::current();
 }
 
@@ -133,6 +140,7 @@ MessageLoop::EventTarget::DispatchFromScript(nsIRunnable* aEvent,
 NS_IMETHODIMP
 MessageLoop::EventTarget::Dispatch(already_AddRefed<nsIRunnable> aEvent,
                                    uint32_t aFlags) {
+  mozilla::MutexAutoLock lock(mMutex);
   if (!mLoop) {
     return NS_ERROR_NOT_INITIALIZED;
   }
@@ -148,6 +156,7 @@ MessageLoop::EventTarget::Dispatch(already_AddRefed<nsIRunnable> aEvent,
 NS_IMETHODIMP
 MessageLoop::EventTarget::DelayedDispatch(already_AddRefed<nsIRunnable> aEvent,
                                           uint32_t aDelayMs) {
+  mozilla::MutexAutoLock lock(mMutex);
   if (!mLoop) {
     return NS_ERROR_NOT_INITIALIZED;
   }
@@ -244,6 +253,13 @@ MessageLoop::MessageLoop(Type type, nsIEventTarget* aEventTarget)
     pump_ = new base::MessagePumpDefault();
   }
 #endif    // OS_POSIX
+
+  // We want GetCurrentSerialEventTarget() to return the real nsThread if it
+  // will be used to dispatch tasks. However, under all other cases; we'll want
+  // it to return this MessageLoop's EventTarget.
+  if (!pump_->GetXPCOMThread()) {
+    mozilla::SerialEventTargetGuard::Set(mEventTarget);
+  }
 }
 
 MessageLoop::~MessageLoop() {
@@ -324,7 +340,7 @@ bool MessageLoop::ProcessNextDelayedNonNestableTask() {
   if (deferred_non_nestable_work_queue_.empty()) return false;
 
   nsCOMPtr<nsIRunnable> task =
-      deferred_non_nestable_work_queue_.front().task.forget();
+      std::move(deferred_non_nestable_work_queue_.front().task);
   deferred_non_nestable_work_queue_.pop();
 
   RunTask(task.forget());
@@ -356,6 +372,7 @@ void MessageLoop::PostIdleTask(already_AddRefed<nsIRunnable> task) {
   MOZ_ASSERT(NS_IsMainThread());
 
   PendingTask pending_task(std::move(task), false);
+  mozilla::LogRunnable::LogDispatch(pending_task.task.get());
   deferred_non_nestable_work_queue_.push(std::move(pending_task));
 }
 
@@ -376,17 +393,7 @@ void MessageLoop::PostTask_Helper(already_AddRefed<nsIRunnable> task,
   // Tasks should only be queued before or during the Run loop, not after.
   MOZ_ASSERT(!shutting_down_);
 
-#ifdef MOZ_TASK_TRACER
-  nsCOMPtr<nsIRunnable> tracedTask = task;
-  if (mozilla::tasktracer::IsStartLogging()) {
-    tracedTask = mozilla::tasktracer::CreateTracedRunnable(tracedTask.forget());
-    (static_cast<mozilla::tasktracer::TracedRunnable*>(tracedTask.get()))
-        ->DispatchTask();
-  }
-  PendingTask pending_task(tracedTask.forget(), true);
-#else
   PendingTask pending_task(std::move(task), true);
-#endif
 
   if (delay_ms > 0) {
     pending_task.delayed_run_time =
@@ -402,6 +409,7 @@ void MessageLoop::PostTask_Helper(already_AddRefed<nsIRunnable> task,
   RefPtr<base::MessagePump> pump;
   {
     mozilla::MutexAutoLock locked(incoming_queue_lock_);
+    mozilla::LogRunnable::LogDispatch(pending_task.task.get());
     incoming_queue_.push(std::move(pending_task));
     pump = pump_;
   }
@@ -439,8 +447,13 @@ void MessageLoop::RunTask(already_AddRefed<nsIRunnable> aTask) {
   nestable_tasks_allowed_ = false;
 
   nsCOMPtr<nsIRunnable> task = aTask;
-  task->Run();
-  task = nullptr;
+
+  {
+    mozilla::LogRunnable::Run log(task.get());
+    AUTO_PROFILE_FOLLOWING_RUNNABLE(task);
+    task->Run();
+    task = nullptr;
+  }
 
   nestable_tasks_allowed_ = true;
 }
@@ -455,6 +468,7 @@ bool MessageLoop::DeferOrRunPendingTask(PendingTask&& pending_task) {
 
   // We couldn't run the task now because we're in a nested message loop
   // and the task isn't nestable.
+  mozilla::LogRunnable::LogDispatch(pending_task.task.get());
   deferred_non_nestable_work_queue_.push(std::move(pending_task));
   return false;
 }
@@ -466,6 +480,7 @@ void MessageLoop::AddToDelayedWorkQueue(const PendingTask& pending_task) {
   // delayed_run_time value.
   PendingTask new_pending_task(pending_task);
   new_pending_task.sequence_num = next_sequence_num_++;
+  mozilla::LogRunnable::LogDispatch(new_pending_task.task.get());
   delayed_work_queue_.push(std::move(new_pending_task));
 }
 
