@@ -26,14 +26,7 @@
 static_assert(MOZ_ALIGNOF(Pickle::memberAlignmentType) >= MOZ_ALIGNOF(uint32_t),
               "Insufficient alignment");
 
-#ifndef MOZ_TASK_TRACER
 static const uint32_t kHeaderSegmentCapacity = 64;
-#else
-// TaskTracer would add extra fields to the header to carry task ID and
-// other information.
-// \see class Message::HeaderTaskTracer
-static const uint32_t kHeaderSegmentCapacity = 128;
-#endif
 
 static const uint32_t kDefaultSegmentCapacity = 4096;
 
@@ -55,12 +48,9 @@ namespace {
 // needs that alignment or less, and memcpy otherwise.  (The compiler
 // may still inline memcpy, of course.)
 
-template<typename T, size_t size, bool hasSufficientAlignment>
-struct Copier
-{
-  static void Copy(T* dest, const char* iter) {
-    memcpy(dest, iter, sizeof(T));
-  }
+template <typename T, size_t size, bool hasSufficientAlignment>
+struct Copier {
+  static void Copy(T* dest, const char* iter) { memcpy(dest, iter, sizeof(T)); }
 };
 
 // Copying 64-bit quantities happens often enough and can easily be made
@@ -68,15 +58,14 @@ struct Copier
 // if 64-bit types aren't sufficiently aligned; the alignment
 // requirements for them vary between 32-bit platforms.
 #ifndef HAVE_64BIT_BUILD
-template<typename T>
-struct Copier<T, sizeof(uint64_t), false>
-{
+template <typename T>
+struct Copier<T, sizeof(uint64_t), false> {
   static void Copy(T* dest, const char* iter) {
-#if MOZ_LITTLE_ENDIAN
+#  if MOZ_LITTLE_ENDIAN
     static const int loIndex = 0, hiIndex = 1;
-#else
+#  else
     static const int loIndex = 1, hiIndex = 0;
-#endif
+#  endif
     static_assert(MOZ_ALIGNOF(uint32_t*) == MOZ_ALIGNOF(void*),
                   "Pointers have different alignments");
     const uint32_t* src = reinterpret_cast<const uint32_t*>(iter);
@@ -87,36 +76,42 @@ struct Copier<T, sizeof(uint64_t), false>
 };
 #endif
 
-template<typename T, size_t size>
-struct Copier<T, size, true>
-{
+template <typename T, size_t size>
+struct Copier<T, size, true> {
   static void Copy(T* dest, const char* iter) {
+    // The pointer ought to be properly aligned.
+    DCHECK_EQ((((uintptr_t)iter) & (MOZ_ALIGNOF(T) - 1)), 0);
     *dest = *reinterpret_cast<const T*>(iter);
   }
 };
 
-} // anonymous namespace
+}  // anonymous namespace
 
 PickleIterator::PickleIterator(const Pickle& pickle)
-   : iter_(pickle.buffers_.Iter())
-   , start_(mozilla::TimeStamp::Now()) {
+    : iter_(pickle.buffers_.Iter()), start_(mozilla::TimeStamp::Now()) {
   iter_.Advance(pickle.buffers_, pickle.header_size_);
 }
 
-template<typename T>
-void
-PickleIterator::CopyInto(T* dest) {
+template <typename T>
+void PickleIterator::CopyInto(T* dest) {
   static_assert(mozilla::IsPod<T>::value, "Copied type must be a POD type");
-  Copier<T, sizeof(T), (MOZ_ALIGNOF(T) <= sizeof(Pickle::memberAlignmentType))>::Copy(dest, iter_.Data());
+  Copier<T, sizeof(T),
+         (MOZ_ALIGNOF(T) <=
+          sizeof(Pickle::memberAlignmentType))>::Copy(dest, iter_.Data());
 }
 
-bool Pickle::IteratorHasRoomFor(const PickleIterator& iter, uint32_t len) const {
+bool Pickle::IteratorHasRoomFor(const PickleIterator& iter,
+                                uint32_t len) const {
   // Make sure we don't get into trouble where AlignInt(len) == 0.
   MOZ_RELEASE_ASSERT(len < 64);
 
   return iter.iter_.HasRoomFor(AlignInt(len));
 }
 
+bool Pickle::HasBytesAvailable(const PickleIterator* iter, uint32_t len) const {
+  return iter->iter_.HasBytesAvailable(buffers_, len);
+}
+
 void Pickle::UpdateIter(PickleIterator* iter, uint32_t bytes) const {
   // Make sure we don't get into trouble where AlignInt(bytes) == 0.
   MOZ_RELEASE_ASSERT(bytes < 64);
@@ -151,32 +146,39 @@ Pickle::Pickle(uint32_t header_size, const char* data, uint32_t length)
 }
 
 Pickle::Pickle(Pickle&& other)
-   : buffers_(mozilla::Move(other.buffers_)),
-     header_(other.header_),
-     header_size_(other.header_size_) {
+    : buffers_(std::move(other.buffers_)),
+      header_(other.header_),
+      header_size_(other.header_size_) {
   other.header_ = nullptr;
 }
 
-Pickle::~Pickle() {
-}
+Pickle::~Pickle() {}
 
 Pickle& Pickle::operator=(Pickle&& other) {
-  BufferList tmp = mozilla::Move(other.buffers_);
-  other.buffers_ = mozilla::Move(buffers_);
-  buffers_ = mozilla::Move(tmp);
+  BufferList tmp = std::move(other.buffers_);
+  other.buffers_ = std::move(buffers_);
+  buffers_ = std::move(tmp);
 
-  //std::swap(buffers_, other.buffers_);
+  // std::swap(buffers_, other.buffers_);
   std::swap(header_, other.header_);
   std::swap(header_size_, other.header_size_);
   return *this;
 }
 
+void Pickle::CopyFrom(const Pickle& other) {
+  MOZ_ALWAYS_TRUE(buffers_.CopyFrom(other.buffers_));
+  MOZ_ASSERT(other.header_ ==
+             reinterpret_cast<const Header*>(other.buffers_.Start()));
+
+  header_ = reinterpret_cast<Header*>(buffers_.Start());
+  header_size_ = other.header_size_;
+}
+
 bool Pickle::ReadBool(PickleIterator* iter, bool* result) const {
   DCHECK(iter);
 
   int tmp;
-  if (!ReadInt(iter, &tmp))
-    return false;
+  if (!ReadInt(iter, &tmp)) return false;
   DCHECK(0 == tmp || 1 == tmp);
   *result = tmp ? true : false;
   return true;
@@ -259,31 +261,10 @@ bool Pickle::ReadULong(PickleIterator* iter, unsigned long* result) const {
 }
 
 bool Pickle::ReadLength(PickleIterator* iter, int* result) const {
-  if (!ReadInt(iter, result))
-    return false;
+  if (!ReadInt(iter, result)) return false;
   return ((*result) >= 0);
 }
 
-// Always written as a 64-bit value since the size for this type can
-// differ between architectures.
-bool Pickle::ReadSize(PickleIterator* iter, size_t* result) const {
-  DCHECK(iter);
-
-  uint64_t big_result = 0;
-  if (IteratorHasRoomFor(*iter, sizeof(big_result))) {
-    iter->CopyInto(&big_result);
-    UpdateIter(iter, sizeof(big_result));
-  } else {
-    if (!ReadBytesInto(iter, &big_result, sizeof(big_result))) {
-      return false;
-    }
-  }
-  DCHECK(big_result <= std::numeric_limits<size_t>::max());
-  *result = static_cast<size_t>(big_result);
-
-  return true;
-}
-
 bool Pickle::ReadInt32(PickleIterator* iter, int32_t* result) const {
   DCHECK(iter);
 
@@ -359,13 +340,15 @@ bool Pickle::ReadIntPtr(PickleIterator* iter, intptr_t* result) const {
     }
   }
 
-  DCHECK(big_result <= std::numeric_limits<intptr_t>::max() && big_result >= std::numeric_limits<intptr_t>::min());
+  DCHECK(big_result <= std::numeric_limits<intptr_t>::max() &&
+         big_result >= std::numeric_limits<intptr_t>::min());
   *result = static_cast<intptr_t>(big_result);
 
   return true;
 }
 
-bool Pickle::ReadUnsignedChar(PickleIterator* iter, unsigned char* result) const {
+bool Pickle::ReadUnsignedChar(PickleIterator* iter,
+                              unsigned char* result) const {
   DCHECK(iter);
 
   if (!IteratorHasRoomFor(*iter, sizeof(*result)))
@@ -381,8 +364,7 @@ bool Pickle::ReadString(PickleIterator* iter, std::string* result) const {
   DCHECK(iter);
 
   int len;
-  if (!ReadLength(iter, &len))
-    return false;
+  if (!ReadLength(iter, &len)) return false;
 
   auto chars = mozilla::MakeUnique<char[]>(len);
   if (!ReadBytesInto(iter, chars.get(), len)) {
@@ -397,11 +379,9 @@ bool Pickle::ReadWString(PickleIterator* iter, std::wstring* result) const {
   DCHECK(iter);
 
   int len;
-  if (!ReadLength(iter, &len))
-    return false;
+  if (!ReadLength(iter, &len)) return false;
   // Avoid integer multiplication overflow.
-  if (len > INT_MAX / static_cast<int>(sizeof(wchar_t)))
-    return false;
+  if (len > INT_MAX / static_cast<int>(sizeof(wchar_t))) return false;
 
   auto chars = mozilla::MakeUnique<wchar_t[]>(len);
   if (!ReadBytesInto(iter, chars.get(), len * sizeof(wchar_t))) {
@@ -412,15 +392,14 @@ bool Pickle::ReadWString(PickleIterator* iter, std::wstring* result) const {
   return true;
 }
 
-bool Pickle::ExtractBuffers(PickleIterator* iter, size_t length, BufferList* buffers,
-                            uint32_t alignment) const
-{
+bool Pickle::ExtractBuffers(PickleIterator* iter, size_t length,
+                            BufferList* buffers, uint32_t alignment) const {
   DCHECK(iter);
   DCHECK(buffers);
   DCHECK(alignment == 4 || alignment == 8);
   DCHECK(intptr_t(header_) % alignment == 0);
 
-  if (AlignInt(length) < length) {
+  if (AlignInt(length) < length || iter->iter_.Done()) {
     return false;
   }
 
@@ -430,7 +409,8 @@ bool Pickle::ExtractBuffers(PickleIterator* iter, size_t length, BufferList* buf
   }
 
   bool success;
-  *buffers = const_cast<BufferList*>(&buffers_)->Extract(iter->iter_, length, &success);
+  *buffers = const_cast<BufferList*>(&buffers_)->Extract(iter->iter_, length,
+                                                         &success);
   if (!success) {
     return false;
   }
@@ -438,7 +418,8 @@ bool Pickle::ExtractBuffers(PickleIterator* iter, size_t length, BufferList* buf
   return iter->iter_.AdvanceAcrossSegments(buffers_, AlignInt(length) - length);
 }
 
-bool Pickle::ReadBytesInto(PickleIterator* iter, void* data, uint32_t length) const {
+bool Pickle::ReadBytesInto(PickleIterator* iter, void* data,
+                           uint32_t length) const {
   if (AlignInt(length) < length) {
     return false;
   }
@@ -450,7 +431,16 @@ bool Pickle::ReadBytesInto(PickleIterator* iter, void* data, uint32_t length) co
   return iter->iter_.AdvanceAcrossSegments(buffers_, AlignInt(length) - length);
 }
 
+bool Pickle::IgnoreBytes(PickleIterator* iter, uint32_t length) const {
+  if (AlignInt(length) < length) {
+    return false;
+  }
+
+  return iter->iter_.AdvanceAcrossSegments(buffers_, AlignInt(length));
+}
+
 #ifdef MOZ_PICKLE_SENTINEL_CHECKING
+MOZ_NEVER_INLINE
 bool Pickle::ReadSentinel(PickleIterator* iter, uint32_t sentinel) const {
   uint32_t found;
   if (!ReadUInt32(iter, &found)) {
@@ -464,24 +454,30 @@ bool Pickle::IgnoreSentinel(PickleIterator* iter) const {
   return ReadUInt32(iter, &found);
 }
 
-bool Pickle::WriteSentinel(uint32_t sentinel) {
-  return WriteUInt32(sentinel);
-}
+bool Pickle::WriteSentinel(uint32_t sentinel) { return WriteUInt32(sentinel); }
 #endif
 
 void Pickle::EndRead(PickleIterator& iter, uint32_t ipcMsgType) const {
-  DCHECK(iter.iter_.Done());
+  // FIXME: Deal with the footer somehow...
+  // DCHECK(iter.iter_.Done());
 
   if (NS_IsMainThread() && ipcMsgType != 0) {
-    uint32_t latencyMs = round((mozilla::TimeStamp::Now() - iter.start_).ToMilliseconds());
+    uint32_t latencyMs =
+        round((mozilla::TimeStamp::Now() - iter.start_).ToMilliseconds());
     if (latencyMs >= kMinTelemetryIPCReadLatencyMs) {
-      mozilla::Telemetry::Accumulate(mozilla::Telemetry::IPC_READ_MAIN_THREAD_LATENCY_MS,
-                                     nsDependentCString(IPC::StringFromIPCMessageType(ipcMsgType)),
-                                     latencyMs);
+      mozilla::Telemetry::Accumulate(
+          mozilla::Telemetry::IPC_READ_MAIN_THREAD_LATENCY_MS,
+          nsDependentCString(IPC::StringFromIPCMessageType(ipcMsgType)),
+          latencyMs);
     }
   }
 }
 
+void Pickle::Truncate(PickleIterator* iter) {
+  size_t dropped = buffers_.Truncate(iter->iter_);
+  header_->payload_size -= dropped;
+}
+
 void Pickle::BeginWrite(uint32_t length, uint32_t alignment) {
   DCHECK(alignment % 4 == 0) << "Must be at least 32-bit aligned!";
 
@@ -500,10 +496,11 @@ void Pickle::BeginWrite(uint32_t length, uint32_t alignment) {
   if (padding) {
     MOZ_RELEASE_ASSERT(padding <= 8);
     static const char padding_data[8] = {
-      kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker,
-      kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker,
+        kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker,
+        kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker,
+        kBytePaddingMarker, kBytePaddingMarker,
     };
-    buffers_.WriteBytes(padding_data, padding);
+    MOZ_ALWAYS_TRUE(buffers_.WriteBytes(padding_data, padding));
   }
 
   DCHECK((header_size_ + header_->payload_size + padding) % alignment == 0);
@@ -518,36 +515,39 @@ void Pickle::EndWrite(uint32_t length) {
   if (padding) {
     MOZ_RELEASE_ASSERT(padding <= 4);
     static const char padding_data[4] = {
-      kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker, kBytePaddingMarker,
+        kBytePaddingMarker,
+        kBytePaddingMarker,
+        kBytePaddingMarker,
+        kBytePaddingMarker,
     };
-    buffers_.WriteBytes(padding_data, padding);
+    MOZ_ALWAYS_TRUE(buffers_.WriteBytes(padding_data, padding));
   }
 }
 
 bool Pickle::WriteBool(bool value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzBool(&value);
+  mozilla::ipc::Faulty::instance().FuzzBool(&value);
 #endif
   return WriteInt(value ? 1 : 0);
 }
 
 bool Pickle::WriteInt16(int16_t value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzInt16(&value);
+  mozilla::ipc::Faulty::instance().FuzzInt16(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
 bool Pickle::WriteUInt16(uint16_t value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzUInt16(&value);
+  mozilla::ipc::Faulty::instance().FuzzUInt16(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
 bool Pickle::WriteInt(int value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzInt(&value);
+  mozilla::ipc::Faulty::instance().FuzzInt(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
@@ -556,7 +556,7 @@ bool Pickle::WriteLong(long value) {
   // Always written as a 64-bit value since the size for this type can
   // differ between architectures.
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzLong(&value);
+  mozilla::ipc::Faulty::instance().FuzzLong(&value);
 #endif
   return WriteInt64(int64_t(value));
 }
@@ -565,51 +565,42 @@ bool Pickle::WriteULong(unsigned long value) {
   // Always written as a 64-bit value since the size for this type can
   // differ between architectures.
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzULong(&value);
-#endif
-  return WriteUInt64(uint64_t(value));
-}
-
-bool Pickle::WriteSize(size_t value) {
-  // Always written as a 64-bit value since the size for this type can
-  // differ between architectures.
-#ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzSize(&value);
+  mozilla::ipc::Faulty::instance().FuzzULong(&value);
 #endif
   return WriteUInt64(uint64_t(value));
 }
 
 bool Pickle::WriteInt32(int32_t value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzInt(&value);
+  mozilla::ipc::Faulty::instance().FuzzInt(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
 bool Pickle::WriteUInt32(uint32_t value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzUInt32(&value);
+  mozilla::ipc::Faulty::instance().FuzzUInt32(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
 bool Pickle::WriteInt64(int64_t value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzInt64(&value);
+  mozilla::ipc::Faulty::instance().FuzzInt64(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
 bool Pickle::WriteUInt64(uint64_t value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzUInt64(&value);
+  mozilla::ipc::Faulty::instance().FuzzUInt64(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
 bool Pickle::WriteDouble(double value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzDouble(&value);
+  mozilla::ipc::Faulty::instance().FuzzDouble(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
@@ -622,28 +613,45 @@ bool Pickle::WriteIntPtr(intptr_t value) {
 
 bool Pickle::WriteUnsignedChar(unsigned char value) {
 #ifdef FUZZING
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzUChar(&value);
+  mozilla::ipc::Faulty::instance().FuzzUChar(&value);
 #endif
   return WriteBytes(&value, sizeof(value));
 }
 
-bool Pickle::WriteBytesZeroCopy(void* data, uint32_t data_len, uint32_t capacity) {
-
+bool Pickle::WriteBytesZeroCopy(void* data, uint32_t data_len,
+                                uint32_t capacity) {
   BeginWrite(data_len, sizeof(memberAlignmentType));
 
-  buffers_.WriteBytesZeroCopy(reinterpret_cast<char*>(data), data_len, capacity);
+  uint32_t new_capacity = AlignInt(capacity);
+#ifndef MOZ_MEMORY
+  if (new_capacity > capacity) {
+    // If the buffer we were given is not large enough to contain padding
+    // after the data, reallocate it to make it so. When using jemalloc,
+    // we're guaranteed the buffer size is going to be at least 4-bytes
+    // aligned, so we skip realloc altogether. Even with other allocators,
+    // the realloc is likely not necessary, but we don't take chances.
+    // At least with ASan, it does matter to realloc to inform ASan we're
+    // going to use more data from the buffer (and let it actually realloc
+    // if it needs to).
+    data = realloc(data, new_capacity);
+  }
+#endif
+  buffers_.WriteBytesZeroCopy(reinterpret_cast<char*>(data), data_len,
+                              new_capacity);
 
   EndWrite(data_len);
   return true;
 }
 
-bool Pickle::WriteBytes(const void* data, uint32_t data_len, uint32_t alignment) {
+bool Pickle::WriteBytes(const void* data, uint32_t data_len,
+                        uint32_t alignment) {
   DCHECK(alignment == 4 || alignment == 8);
   DCHECK(intptr_t(header_) % alignment == 0);
 
   BeginWrite(data_len, alignment);
 
-  buffers_.WriteBytes(reinterpret_cast<const char*>(data), data_len);
+  MOZ_ALWAYS_TRUE(
+      buffers_.WriteBytes(reinterpret_cast<const char*>(data), data_len));
 
   EndWrite(data_len);
   return true;
@@ -652,14 +660,12 @@ bool Pickle::WriteBytes(const void* data, uint32_t data_len, uint32_t alignment)
 bool Pickle::WriteString(const std::string& value) {
 #ifdef FUZZING
   std::string v(value);
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzString(v);
-  if (!WriteInt(static_cast<int>(v.size())))
-    return false;
+  mozilla::ipc::Faulty::instance().FuzzString(v);
+  if (!WriteInt(static_cast<int>(v.size()))) return false;
 
   return WriteBytes(v.data(), static_cast<int>(v.size()));
 #else
-  if (!WriteInt(static_cast<int>(value.size())))
-    return false;
+  if (!WriteInt(static_cast<int>(value.size()))) return false;
 
   return WriteBytes(value.data(), static_cast<int>(value.size()));
 #endif
@@ -668,15 +674,12 @@ bool Pickle::WriteString(const std::string& value) {
 bool Pickle::WriteWString(const std::wstring& value) {
 #ifdef FUZZING
   std::wstring v(value);
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzWString(v);
-  if (!WriteInt(static_cast<int>(v.size())))
-    return false;
+  mozilla::ipc::Faulty::instance().FuzzWString(v);
+  if (!WriteInt(static_cast<int>(v.size()))) return false;
 
-  return WriteBytes(v.data(),
-                    static_cast<int>(v.size() * sizeof(wchar_t)));
+  return WriteBytes(v.data(), static_cast<int>(v.size() * sizeof(wchar_t)));
 #else
-  if (!WriteInt(static_cast<int>(value.size())))
-    return false;
+  if (!WriteInt(static_cast<int>(value.size()))) return false;
 
   return WriteBytes(value.data(),
                     static_cast<int>(value.size() * sizeof(wchar_t)));
@@ -684,17 +687,11 @@ bool Pickle::WriteWString(const std::wstring& value) {
 }
 
 bool Pickle::WriteData(const char* data, uint32_t length) {
-#ifdef FUZZING
-  std::string v(data, length);
-  Singleton<mozilla::ipc::Faulty>::get()->FuzzData(v, v.size());
-  return WriteInt(v.size()) && WriteBytes(v.data(), v.size());
-#else
-   return WriteInt(length) && WriteBytes(data, length);
-#endif
+  return WriteInt(length) && WriteBytes(data, length);
 }
 
 void Pickle::InputBytes(const char* data, uint32_t length) {
-  buffers_.WriteBytes(data, length);
+  MOZ_ALWAYS_TRUE(buffers_.WriteBytes(data, length));
 }
 
 int32_t* Pickle::GetInt32PtrForTest(uint32_t offset) {
@@ -705,27 +702,23 @@ int32_t* Pickle::GetInt32PtrForTest(uint32_t offset) {
 }
 
 // static
-uint32_t Pickle::MessageSize(uint32_t header_size,
-                             const char* start,
+uint32_t Pickle::MessageSize(uint32_t header_size, const char* start,
                              const char* end) {
   DCHECK(header_size == AlignInt(header_size));
-  DCHECK(header_size <= static_cast<memberAlignmentType>(kHeaderSegmentCapacity));
+  DCHECK(header_size <=
+         static_cast<memberAlignmentType>(kHeaderSegmentCapacity));
 
-  if (end < start)
-    return 0;
+  if (end < start) return 0;
   size_t length = static_cast<size_t>(end - start);
-  if (length < sizeof(Header))
-    return 0;
+  if (length < sizeof(Header)) return 0;
 
   const Header* hdr = reinterpret_cast<const Header*>(start);
-  if (length < header_size)
-    return 0;
+  if (length < header_size) return 0;
 
   mozilla::CheckedInt<uint32_t> sum(header_size);
   sum += hdr->payload_size;
 
-  if (!sum.isValid())
-    return 0;
+  if (!sum.isValid()) return 0;
 
   return sum.value();
 }
