@@ -20,6 +20,12 @@ class ContentParent;
 
 }  // namespace dom
 
+namespace net {
+
+class SocketProcessParent;
+
+}  // namespace net
+
 namespace ipc {
 
 class BackgroundParentImpl;
@@ -28,6 +34,7 @@ class FileDescriptor;
 class FileDescriptorSetParent final : public PFileDescriptorSetParent {
   friend class BackgroundParentImpl;
   friend class mozilla::dom::ContentParent;
+  friend class mozilla::net::SocketProcessParent;
   friend class PFileDescriptorSetParent;
 
   nsTArray<FileDescriptor> mFileDescriptors;
