@@ -7,6 +7,7 @@
 #include "IPCStreamUtils.h"
 
 #include "nsIIPCSerializableInputStream.h"
+#include "mozIRemoteLazyInputStream.h"
 
 #include "mozilla/Assertions.h"
 #include "mozilla/dom/ContentChild.h"
@@ -15,10 +16,14 @@
 #include "mozilla/ipc/FileDescriptorSetChild.h"
 #include "mozilla/ipc/FileDescriptorSetParent.h"
 #include "mozilla/ipc/InputStreamUtils.h"
-#include "mozilla/ipc/PBackgroundChild.h"
-#include "mozilla/ipc/PBackgroundParent.h"
+#include "mozilla/net/SocketProcessChild.h"
+#include "mozilla/net/SocketProcessParent.h"
+#include "mozilla/InputStreamLengthHelper.h"
+#include "mozilla/RemoteLazyInputStreamParent.h"
 #include "mozilla/Unused.h"
 #include "nsNetCID.h"
+#include "BackgroundParentImpl.h"
+#include "BackgroundChildImpl.h"
 
 using namespace mozilla::dom;
 
@@ -38,6 +43,8 @@ bool SerializeInputStreamWithFdsChild(nsIIPCSerializableInputStream* aStream,
   MOZ_RELEASE_ASSERT(aStream);
   MOZ_ASSERT(aManager);
 
+  // If you change this size, please also update the payload size in
+  // test_reload_large_postdata.html.
   const uint64_t kTooLargeStream = 1024 * 1024;
 
   uint32_t sizeUsed = 0;
@@ -126,6 +133,55 @@ bool SerializeInputStream(nsIInputStream* aStream, IPCStream& aValue,
   return true;
 }
 
+template <typename M>
+bool SerializeLazyInputStream(nsIInputStream* aStream, IPCStream& aValue,
+                              M* aManager) {
+  MOZ_ASSERT(aStream);
+  MOZ_ASSERT(aManager);
+  MOZ_ASSERT(XRE_IsParentProcess());
+
+  // avoid creating a loop between processes by accessing the underlying input
+  // stream.
+  nsCOMPtr<nsIInputStream> stream = aStream;
+  if (nsCOMPtr<mozIRemoteLazyInputStream> remoteLazyInputStream =
+          do_QueryInterface(stream)) {
+    stream = remoteLazyInputStream->GetInternalStream();
+    if (NS_WARN_IF(!stream)) {
+      return false;
+    }
+  }
+
+  uint64_t childID = 0;
+  if constexpr (std::is_same_v<dom::ContentParent, M>) {
+    childID = aManager->ChildID();
+  } else if constexpr (std::is_base_of_v<PBackgroundParent, M>) {
+    childID = BackgroundParent::GetChildID(aManager);
+  }
+
+  int64_t length = -1;
+  if (!InputStreamLengthHelper::GetSyncLength(stream, &length)) {
+    length = -1;
+  }
+
+  nsresult rv = NS_OK;
+  RefPtr<RemoteLazyInputStreamParent> actor =
+      RemoteLazyInputStreamParent::Create(aStream, length, childID, &rv,
+                                          aManager);
+  if (NS_WARN_IF(NS_FAILED(rv))) {
+    return false;
+  }
+
+  if (!aManager->SendPRemoteLazyInputStreamConstructor(actor, actor->ID(),
+                                                       actor->Size())) {
+    return false;
+  }
+
+  aValue.stream() = RemoteLazyInputStreamParams(actor);
+  aValue.optionalFds() = void_t();
+
+  return true;
+}
+
 template <typename M>
 bool SerializeInputStreamChild(nsIInputStream* aStream, M* aManager,
                                IPCStream* aValue,
@@ -165,6 +221,16 @@ bool SerializeInputStreamParent(nsIInputStream* aStream, M* aManager,
   MOZ_ASSERT(aManager);
   MOZ_ASSERT(aValue || aOptionalValue);
 
+  // When requesting a delayed start stream from the parent process, serialize
+  // it as a remote lazy stream to avoid bloating payloads.
+  if (aDelayedStart && XRE_IsParentProcess()) {
+    if (aValue) {
+      return SerializeLazyInputStream(aStream, *aValue, aManager);
+    }
+
+    return SerializeLazyInputStream(aStream, aOptionalValue->ref(), aManager);
+  }
+
   nsCOMPtr<nsIIPCSerializableInputStream> serializable =
       do_QueryInterface(aStream);
 
@@ -302,22 +368,13 @@ already_AddRefed<nsIInputStream> DeserializeIPCStream(
 }
 
 AutoIPCStream::AutoIPCStream(bool aDelayedStart)
-    : mValue(nullptr),
-      mOptionalValue(&mInlineValue),
-      mTaken(false),
-      mDelayedStart(aDelayedStart) {}
+    : mOptionalValue(&mInlineValue), mDelayedStart(aDelayedStart) {}
 
 AutoIPCStream::AutoIPCStream(IPCStream& aTarget, bool aDelayedStart)
-    : mValue(&aTarget),
-      mOptionalValue(nullptr),
-      mTaken(false),
-      mDelayedStart(aDelayedStart) {}
+    : mValue(&aTarget), mDelayedStart(aDelayedStart) {}
 
 AutoIPCStream::AutoIPCStream(Maybe<IPCStream>& aTarget, bool aDelayedStart)
-    : mValue(nullptr),
-      mOptionalValue(&aTarget),
-      mTaken(false),
-      mDelayedStart(aDelayedStart) {
+    : mOptionalValue(&aTarget), mDelayedStart(aDelayedStart) {
   mOptionalValue->reset();
 }
 
@@ -364,6 +421,28 @@ bool AutoIPCStream::Serialize(nsIInputStream* aStream,
     return true;
   }
 
+  BackgroundChildImpl* impl = static_cast<BackgroundChildImpl*>(aManager);
+  if (!SerializeInputStreamChild(aStream, impl, mValue, mOptionalValue,
+                                 mDelayedStart)) {
+    MOZ_CRASH("IPCStream creation failed!");
+  }
+
+  return true;
+}
+
+bool AutoIPCStream::Serialize(nsIInputStream* aStream,
+                              net::SocketProcessChild* aManager) {
+  MOZ_ASSERT(aStream || !mValue);
+  MOZ_ASSERT(aManager);
+  MOZ_ASSERT(mValue || mOptionalValue);
+  MOZ_ASSERT(!mTaken);
+  MOZ_ASSERT(!IsSet());
+
+  // If NormalizeOptionalValue returns false, we don't have to proceed.
+  if (!NormalizeOptionalValue(aStream, mValue, mOptionalValue)) {
+    return true;
+  }
+
   if (!SerializeInputStreamChild(aStream, aManager, mValue, mOptionalValue,
                                  mDelayedStart)) {
     MOZ_CRASH("IPCStream creation failed!");
@@ -406,6 +485,28 @@ bool AutoIPCStream::Serialize(nsIInputStream* aStream,
     return true;
   }
 
+  BackgroundParentImpl* impl = static_cast<BackgroundParentImpl*>(aManager);
+  if (!SerializeInputStreamParent(aStream, impl, mValue, mOptionalValue,
+                                  mDelayedStart)) {
+    return false;
+  }
+
+  return true;
+}
+
+bool AutoIPCStream::Serialize(nsIInputStream* aStream,
+                              net::SocketProcessParent* aManager) {
+  MOZ_ASSERT(aStream || !mValue);
+  MOZ_ASSERT(aManager);
+  MOZ_ASSERT(mValue || mOptionalValue);
+  MOZ_ASSERT(!mTaken);
+  MOZ_ASSERT(!IsSet());
+
+  // If NormalizeOptionalValue returns false, we don't have to proceed.
+  if (!NormalizeOptionalValue(aStream, mValue, mOptionalValue)) {
+    return true;
+  }
+
   if (!SerializeInputStreamParent(aStream, aManager, mValue, mOptionalValue,
                                   mDelayedStart)) {
     return false;
@@ -447,10 +548,11 @@ Maybe<IPCStream>& AutoIPCStream::TakeOptionalValue() {
   return *mOptionalValue;
 }
 
-void IPDLParamTraits<nsIInputStream>::Write(IPC::Message* aMsg,
-                                            IProtocol* aActor,
-                                            nsIInputStream* aParam) {
-  mozilla::ipc::AutoIPCStream autoStream;
+void IPDLParamTraits<nsIInputStream*>::Write(IPC::Message* aMsg,
+                                             IProtocol* aActor,
+                                             nsIInputStream* aParam) {
+  auto autoStream = MakeRefPtr<HoldIPCStream>(/* aDelayedStart */ true);
+
   bool ok = false;
   bool found = false;
 
@@ -459,29 +561,31 @@ void IPDLParamTraits<nsIInputStream>::Write(IPC::Message* aMsg,
   // protocols we support.
   IProtocol* actor = aActor;
   while (!found && actor) {
-    switch (actor->GetProtocolTypeId()) {
+    switch (actor->GetProtocolId()) {
       case PContentMsgStart:
         if (actor->GetSide() == mozilla::ipc::ParentSide) {
-          ok = autoStream.Serialize(
+          ok = autoStream->Serialize(
               aParam, static_cast<mozilla::dom::ContentParent*>(actor));
         } else {
           MOZ_RELEASE_ASSERT(actor->GetSide() == mozilla::ipc::ChildSide);
-          ok = autoStream.Serialize(
+          ok = autoStream->Serialize(
               aParam, static_cast<mozilla::dom::ContentChild*>(actor));
         }
         found = true;
         break;
       case PBackgroundMsgStart:
         if (actor->GetSide() == mozilla::ipc::ParentSide) {
-          ok = autoStream.Serialize(
+          ok = autoStream->Serialize(
               aParam, static_cast<mozilla::ipc::PBackgroundParent*>(actor));
         } else {
           MOZ_RELEASE_ASSERT(actor->GetSide() == mozilla::ipc::ChildSide);
-          ok = autoStream.Serialize(
+          ok = autoStream->Serialize(
               aParam, static_cast<mozilla::ipc::PBackgroundChild*>(actor));
         }
         found = true;
         break;
+      default:
+        break;
     }
 
     // Try the actor's manager.
@@ -494,13 +598,19 @@ void IPDLParamTraits<nsIInputStream>::Write(IPC::Message* aMsg,
   }
   MOZ_RELEASE_ASSERT(ok, "Failed to serialize nsIInputStream");
 
-  WriteIPDLParam(aMsg, aActor, autoStream.TakeOptionalValue());
+  WriteIPDLParam(aMsg, aActor, autoStream->TakeOptionalValue());
+
+  // Dispatch the autoStream to an async runnable, so that we guarantee it
+  // outlives this callstack, and doesn't shut down any actors we created
+  // until after we've finished sending the current message.
+  NS_ProxyRelease("IPDLParamTraits<nsIInputStream*>::Write::autoStream",
+                  NS_GetCurrentThread(), autoStream.forget(), true);
 }
 
-bool IPDLParamTraits<nsIInputStream>::Read(const IPC::Message* aMsg,
-                                           PickleIterator* aIter,
-                                           IProtocol* aActor,
-                                           RefPtr<nsIInputStream>* aResult) {
+bool IPDLParamTraits<nsIInputStream*>::Read(const IPC::Message* aMsg,
+                                            PickleIterator* aIter,
+                                            IProtocol* aActor,
+                                            RefPtr<nsIInputStream>* aResult) {
   mozilla::Maybe<mozilla::ipc::IPCStream> ipcStream;
   if (!ReadIPDLParam(aMsg, aIter, aActor, &ipcStream)) {
     return false;
