@@ -13,18 +13,19 @@ from ipdl.ast import *
 class ParseError(Exception):
     def __init__(self, loc, fmt, *args):
         self.loc = loc
-        self.error = ('%s%s: error: %s' % (
-            Parser.includeStackString(), loc, fmt)) % args
+        self.error = (
+            "%s%s: error: %s" % (Parser.includeStackString(), loc, fmt)
+        ) % args
 
     def __str__(self):
         return self.error
 
 
 def _safeLinenoValue(t):
-    lineno, value = 0, '???'
-    if hasattr(t, 'lineno'):
+    lineno, value = 0, "???"
+    if hasattr(t, "lineno"):
         lineno = t.lineno
-    if hasattr(t, 'value'):
+    if hasattr(t, "value"):
         value = t.value
     return lineno, value
 
@@ -49,7 +50,7 @@ class Parser:
         self.debug = debug
         self.filename = None
         self.includedirs = None
-        self.loc = None         # not always up to date
+        self.loc = None  # not always up to date
         self.lexer = None
         self.parser = None
         self.tu = TranslationUnit(type, name)
@@ -58,35 +59,38 @@ class Parser:
     def parse(self, input, filename, includedirs):
         assert os.path.isabs(filename)
 
-        if filename in Parser.parsed:
-            return Parser.parsed[filename].tu
+        if self.tu.name in Parser.parsed:
+            priorTU = Parser.parsed[self.tu.name].tu
+            if priorTU.filename != filename:
+                _error(
+                    Loc(filename),
+                    "Trying to load `%s' from a file when we'd already seen it in file `%s'"
+                    % (self.tu.name, priorTU.filename),
+                )
 
-        self.lexer = lex.lex(debug=self.debug,
-                             optimize=not self.debug,
-                             lextab="ipdl_lextab")
-        self.parser = yacc.yacc(debug=self.debug,
-                                optimize=not self.debug,
-                                tabmodule="ipdl_yacctab")
+            return priorTU
+
+        self.lexer = lex.lex(debug=self.debug)
+        self.parser = yacc.yacc(debug=self.debug, write_tables=False)
         self.filename = filename
         self.includedirs = includedirs
         self.tu.filename = filename
 
-        Parser.parsed[filename] = self
+        Parser.parsed[self.tu.name] = self
         Parser.parseStack.append(Parser.current)
         Parser.current = self
 
         try:
-            ast = self.parser.parse(input=input, lexer=self.lexer,
-                                    debug=self.debug)
+            ast = self.parser.parse(input=input, lexer=self.lexer, debug=self.debug)
         finally:
             Parser.current = Parser.parseStack.pop()
 
         return ast
 
     def resolveIncludePath(self, filepath):
-        '''Return the absolute path from which the possibly partial
-|filepath| should be read, or |None| if |filepath| cannot be located.'''
-        for incdir in self.includedirs + ['']:
+        """Return the absolute path from which the possibly partial
+        |filepath| should be read, or |None| if |filepath| cannot be located."""
+        for incdir in self.includedirs + [""]:
             realpath = os.path.join(incdir, filepath)
             if os.path.isfile(realpath):
                 return os.path.abspath(realpath)
@@ -99,10 +103,12 @@ class Parser:
     # which can be printed above a proper error message or warning
     @staticmethod
     def includeStackString():
-        s = ''
+        s = ""
         for parse in Parser.parseStack[1:]:
             s += "  in file included from `%s', line %d:\n" % (
-                parse.loc.filename, parse.loc.lineno)
+                parse.loc.filename,
+                parse.loc.lineno,
+            )
         return s
 
 
@@ -112,61 +118,61 @@ def locFromTok(p, num):
 
 # -----------------------------------------------------------------------------
 
-reserved = set((
-    'async',
-    'both',
-    'child',
-    'class',
-    'compress',
-    'compressall',
-    'from',
-    'include',
-    'intr',
-    'manager',
-    'manages',
-    'namespace',
-    'nested',
-    'nullable',
-    'or',
-    'parent',
-    'prio',
-    'protocol',
-    'refcounted',
-    'moveonly',
-    'returns',
-    'struct',
-    'sync',
-    'union',
-    'UniquePtr',
-    'upto',
-    'using',
-    'verify'))
+reserved = set(
+    (
+        "async",
+        "both",
+        "child",
+        "class",
+        "from",
+        "include",
+        "intr",
+        "manager",
+        "manages",
+        "namespace",
+        "nested",
+        "nullable",
+        "or",
+        "parent",
+        "protocol",
+        "refcounted",
+        "returns",
+        "struct",
+        "sync",
+        "union",
+        "UniquePtr",
+        "upto",
+        "using",
+    )
+)
 tokens = [
-    'COLONCOLON', 'ID', 'STRING',
+    "COLONCOLON",
+    "ID",
+    "STRING",
 ] + [r.upper() for r in reserved]
 
-t_COLONCOLON = '::'
+t_COLONCOLON = "::"
 
-literals = '(){}[]<>;:,?'
-t_ignore = ' \f\t\v'
+literals = "(){}[]<>;:,?="
+t_ignore = " \f\t\v"
 
 
 def t_linecomment(t):
-    r'//[^\n]*'
+    r"//[^\n]*"
 
 
 def t_multilinecomment(t):
-    r'/\*(\n|.)*?\*/'
-    t.lexer.lineno += t.value.count('\n')
+    r"/\*(\n|.)*?\*/"
+    t.lexer.lineno += t.value.count("\n")
 
 
 def t_NL(t):
-    r'(?:\r\n|\n|\n)+'
+    r"(?:\r\n|\n|\n)+"
     t.lexer.lineno += len(t.value)
 
 
 def t_ID(t):
-    r'[a-zA-Z_][a-zA-Z0-9_]*'
+    r"[a-zA-Z_][a-zA-Z0-9_]*"
     if t.value in reserved:
         t.type = t.value.upper()
     return t
@@ -179,8 +185,12 @@ def t_STRING(t):
 
 
 def t_error(t):
-    _error(Loc(Parser.current.filename, t.lineno),
-           'lexically invalid characters `%s', t.value)
+    _error(
+        Loc(Parser.current.filename, t.lineno),
+        "lexically invalid characters `%s",
+        t.value,
+    )
+
 
 # -----------------------------------------------------------------------------
 
@@ -209,16 +219,16 @@ def p_TranslationUnit(p):
                 _error(thing.loc, "only one protocol definition per file")
             tu.protocol = thing
         else:
-            assert(0)
+            assert 0
 
     # The "canonical" namespace of the tu, what it's considered to be
     # in for the purposes of C++: |#include "foo/bar/TU.h"|
     if tu.protocol:
-        assert tu.filetype == 'protocol'
+        assert tu.filetype == "protocol"
         tu.namespaces = tu.protocol.namespaces
         tu.name = tu.protocol.name
     else:
-        assert tu.filetype == 'header'
+        assert tu.filetype == "header"
         # There's not really a canonical "thing" in headers.  So
         # somewhat arbitrarily use the namespace of the last
         # interesting thing that was declared.
@@ -228,13 +238,14 @@ def p_TranslationUnit(p):
 
     p[0] = tu
 
+
 # --------------------
 # Preamble
 
 
 def p_Preamble(p):
     """Preamble : Preamble PreambleStmt ';'
-                |"""
+    |"""
     if 1 == len(p):
         p[0] = []
     else:
@@ -244,8 +255,8 @@ def p_Preamble(p):
 
 def p_PreambleStmt(p):
     """PreambleStmt : CxxIncludeStmt
-                    | IncludeStmt
-                    | UsingStmt"""
+    | IncludeStmt
+    | UsingStmt"""
     p[0] = p[1]
 
 
@@ -256,22 +267,21 @@ def p_CxxIncludeStmt(p):
 
 def p_IncludeStmt(p):
     """IncludeStmt : INCLUDE PROTOCOL ID
-                   | INCLUDE ID"""
+    | INCLUDE ID"""
     loc = locFromTok(p, 1)
 
     Parser.current.loc = loc
     if 4 == len(p):
         id = p[3]
-        type = 'protocol'
+        type = "protocol"
     else:
         id = p[2]
-        type = 'header'
+        type = "header"
     inc = Include(loc, type, id)
 
     path = Parser.current.resolveIncludePath(inc.file)
     if path is None:
-        raise ParseError(loc, "can't locate include file `%s'" % (
-            inc.file))
+        raise ParseError(loc, "can't locate include file `%s'" % (inc.file))
 
     inc.tu = Parser(type, id).parse(open(path).read(), path, Parser.current.includedirs)
     p[0] = inc
@@ -279,31 +289,27 @@ def p_IncludeStmt(p):
 
 def p_UsingKind(p):
     """UsingKind : CLASS
-                 | STRUCT
-                 |"""
+    | STRUCT
+    |"""
     p[0] = p[1] if 2 == len(p) else None
 
 
 def p_MaybeRefcounted(p):
     """MaybeRefcounted : REFCOUNTED
-                       |"""
-    p[0] = 2 == len(p)
-
-
-def p_MaybeMoveOnly(p):
-    """MaybeMoveOnly : MOVEONLY
-                       |"""
+    |"""
     p[0] = 2 == len(p)
 
 
 def p_UsingStmt(p):
-    """UsingStmt : USING MaybeRefcounted MaybeMoveOnly UsingKind CxxType FROM STRING"""
-    p[0] = UsingStmt(locFromTok(p, 1),
-                     refcounted=p[2],
-                     moveonly=p[3],
-                     kind=p[4],
-                     cxxTypeSpec=p[5],
-                     cxxHeader=p[7])
+    """UsingStmt : Attributes USING UsingKind CxxType FROM STRING"""
+    p[0] = UsingStmt(
+        locFromTok(p, 2),
+        attributes=p[1],
+        kind=p[3],
+        cxxTypeSpec=p[4],
+        cxxHeader=p[6],
+    )
+
 
 # --------------------
 # Namespaced stuff
@@ -311,7 +317,7 @@ def p_UsingStmt(p):
 
 def p_NamespacedStuff(p):
     """NamespacedStuff : NamespacedStuff NamespaceThing
-                       | NamespaceThing"""
+    | NamespaceThing"""
     if 2 == len(p):
         p[0] = p[1]
     else:
@@ -321,9 +327,9 @@ def p_NamespacedStuff(p):
 
 def p_NamespaceThing(p):
     """NamespaceThing : NAMESPACE ID '{' NamespacedStuff '}'
-                      | StructDecl
-                      | UnionDecl
-                      | ProtocolDefn"""
+    | StructDecl
+    | UnionDecl
+    | ProtocolDefn"""
     if 2 == len(p):
         p[0] = [p[1]]
     else:
@@ -333,17 +339,17 @@ def p_NamespaceThing(p):
 
 
 def p_StructDecl(p):
-    """StructDecl : STRUCT ID '{' StructFields '}' ';'
-                  | STRUCT ID '{' '}' ';'"""
-    if 7 == len(p):
-        p[0] = StructDecl(locFromTok(p, 1), p[2], p[4])
+    """StructDecl : Attributes STRUCT ID '{' StructFields '}' ';'
+    | Attributes STRUCT ID '{' '}' ';'"""
+    if 8 == len(p):
+        p[0] = StructDecl(locFromTok(p, 2), p[3], p[5], p[1])
     else:
-        p[0] = StructDecl(locFromTok(p, 1), p[2], [])
+        p[0] = StructDecl(locFromTok(p, 2), p[3], [], p[1])
 
 
 def p_StructFields(p):
     """StructFields : StructFields StructField ';'
-                    | StructField ';'"""
+    | StructField ';'"""
     if 3 == len(p):
         p[0] = [p[1]]
     else:
@@ -357,13 +363,13 @@ def p_StructField(p):
 
 
 def p_UnionDecl(p):
-    """UnionDecl : UNION ID '{' ComponentTypes  '}' ';'"""
-    p[0] = UnionDecl(locFromTok(p, 1), p[2], p[4])
+    """UnionDecl : Attributes UNION ID '{' ComponentTypes  '}' ';'"""
+    p[0] = UnionDecl(locFromTok(p, 2), p[3], p[5], p[1])
 
 
 def p_ComponentTypes(p):
     """ComponentTypes : ComponentTypes Type ';'
-                      | Type ';'"""
+    | Type ';'"""
     if 3 == len(p):
         p[0] = [p[1]]
     else:
@@ -372,17 +378,21 @@ def p_ComponentTypes(p):
 
 
 def p_ProtocolDefn(p):
-    """ProtocolDefn : OptionalProtocolSendSemanticsQual PROTOCOL ID '{' ProtocolBody '}' ';'"""
-    protocol = p[5]
-    protocol.loc = locFromTok(p, 2)
-    protocol.name = p[3]
+    """ProtocolDefn : OptionalProtocolSendSemanticsQual MaybeRefcounted \
+                      PROTOCOL ID '{' ProtocolBody '}' ';'"""
+    protocol = p[6]
+    protocol.loc = locFromTok(p, 3)
+    protocol.name = p[4]
     protocol.nested = p[1][0]
     protocol.sendSemantics = p[1][1]
+    protocol.refcounted = p[2]
     p[0] = protocol
 
-    if Parser.current.type == 'header':
-        _error(protocol.loc,
-               'can\'t define a protocol in a header.  Do it in a protocol spec instead.')
+    if Parser.current.type == "header":
+        _error(
+            protocol.loc,
+            "can't define a protocol in a header.  Do it in a protocol spec instead.",
+        )
 
 
 def p_ProtocolBody(p):
@@ -393,9 +403,10 @@ def p_ProtocolBody(p):
 # --------------------
 # manager/manages stmts
 
+
 def p_ManagersStmtOpt(p):
     """ManagersStmtOpt : ManagersStmt ManagesStmtsOpt
-                       | ManagesStmtsOpt"""
+    | ManagesStmtsOpt"""
     if 2 == len(p):
         p[0] = p[1]
     else:
@@ -413,7 +424,7 @@ def p_ManagersStmt(p):
 
 def p_ManagerList(p):
     """ManagerList : ID
-                   | ManagerList OR ID"""
+    | ManagerList OR ID"""
     if 2 == len(p):
         p[0] = [Manager(locFromTok(p, 1), p[1])]
     else:
@@ -423,7 +434,7 @@ def p_ManagerList(p):
 
 def p_ManagesStmtsOpt(p):
     """ManagesStmtsOpt : ManagesStmt ManagesStmtsOpt
-                       | MessageDeclsOpt"""
+    | MessageDeclsOpt"""
     if 2 == len(p):
         p[0] = p[1]
     else:
@@ -439,9 +450,10 @@ def p_ManagesStmt(p):
 # --------------------
 # Message decls
 
+
 def p_MessageDeclsOpt(p):
     """MessageDeclsOpt : MessageDeclThing MessageDeclsOpt
-                       | """
+    |"""
     if 1 == len(p):
         # we fill in |loc| in the Protocol rule
         p[0] = Protocol(None)
@@ -452,7 +464,7 @@ def p_MessageDeclsOpt(p):
 
 def p_MessageDeclThing(p):
     """MessageDeclThing : MessageDirectionLabel ':' MessageDecl ';'
-                        | MessageDecl ';'"""
+    | MessageDecl ';'"""
     if 3 == len(p):
         p[0] = p[1]
     else:
@@ -461,41 +473,39 @@ def p_MessageDeclThing(p):
 
 def p_MessageDirectionLabel(p):
     """MessageDirectionLabel : PARENT
-                             | CHILD
-                             | BOTH"""
-    if p[1] == 'parent':
+    | CHILD
+    | BOTH"""
+    if p[1] == "parent":
         Parser.current.direction = IN
-    elif p[1] == 'child':
+    elif p[1] == "child":
         Parser.current.direction = OUT
-    elif p[1] == 'both':
+    elif p[1] == "both":
         Parser.current.direction = INOUT
     else:
         assert 0
 
 
 def p_MessageDecl(p):
-    """MessageDecl : SendSemanticsQual MessageBody"""
-    msg = p[2]
-    msg.nested = p[1][0]
-    msg.prio = p[1][1]
-    msg.sendSemantics = p[1][2]
+    """MessageDecl : Attributes SendSemantics MessageBody"""
+    msg = p[3]
+    msg.attributes = p[1]
+    msg.sendSemantics = p[2]
 
     if Parser.current.direction is None:
-        _error(msg.loc, 'missing message direction')
+        _error(msg.loc, "missing message direction")
     msg.direction = Parser.current.direction
 
     p[0] = msg
 
 
 def p_MessageBody(p):
-    """MessageBody : ID MessageInParams MessageOutParams OptionalMessageModifiers"""
+    """MessageBody : ID MessageInParams MessageOutParams"""
     # FIXME/cjones: need better loc info: use one of the quals
     name = p[1]
     msg = MessageDecl(locFromTok(p, 1))
     msg.name = name
     msg.addInParams(p[2])
     msg.addOutParams(p[3])
-    msg.addModifiers(p[4])
 
     p[0] = msg
 
@@ -507,109 +517,77 @@ def p_MessageInParams(p):
 
 def p_MessageOutParams(p):
     """MessageOutParams : RETURNS '(' ParamList ')'
-                        | """
+    |"""
     if 1 == len(p):
         p[0] = []
     else:
         p[0] = p[3]
 
 
-def p_OptionalMessageModifiers(p):
-    """OptionalMessageModifiers : OptionalMessageModifiers MessageModifier
-                                | MessageModifier
-                                | """
+# --------------------
+# Attributes
+def p_Attributes(p):
+    """Attributes : '[' AttributeList ']'
+    |"""
+    p[0] = {}
+    if 4 == len(p):
+        for attr in p[2]:
+            if attr.name in p[0]:
+                _error(attr.loc, "Repeated extended attribute `%s'", attr.name)
+            p[0][attr.name] = attr
+
+
+def p_AttributeList(p):
+    """AttributeList : Attribute ',' AttributeList
+    | Attribute"""
+    p[0] = [p[1]]
+    if 4 == len(p):
+        p[0] += p[3]
+
+
+def p_Attribute(p):
+    """Attribute : ID AttributeValue"""
+    p[0] = Attribute(locFromTok(p, 1), p[1], p[2])
+
+
+def p_AttributeValue(p):
+    """AttributeValue : '=' ID
+    |"""
     if 1 == len(p):
-        p[0] = []
-    elif 2 == len(p):
-        p[0] = [p[1]]
+        p[0] = None
     else:
-        p[1].append(p[2])
-        p[0] = p[1]
-
-
-def p_MessageModifier(p):
-    """ MessageModifier : MessageVerify
-                        | MessageCompress """
-    p[0] = p[1]
-
-
-def p_MessageVerify(p):
-    """MessageVerify : VERIFY"""
-    p[0] = p[1]
-
-
-def p_MessageCompress(p):
-    """MessageCompress : COMPRESS
-                       | COMPRESSALL"""
-    p[0] = p[1]
+        p[0] = p[2]
 
 
 # --------------------
 # Minor stuff
 def p_Nested(p):
     """Nested : ID"""
-    kinds = {'not': 1,
-             'inside_sync': 2,
-             'inside_cpow': 3}
+    kinds = {"not": 1, "inside_sync": 2, "inside_cpow": 3}
     if p[1] not in kinds:
-        _error(locFromTok(p, 1), "Expected not, inside_sync, or inside_cpow for nested()")
+        _error(
+            locFromTok(p, 1), "Expected not, inside_sync, or inside_cpow for nested()"
+        )
 
-    p[0] = {'nested': kinds[p[1]]}
+    p[0] = {"nested": kinds[p[1]]}
 
 
-def p_Priority(p):
-    """Priority : ID"""
-    kinds = {'normal': 1,
-             'input': 2,
-             'high': 3}
-    if p[1] not in kinds:
-        _error(locFromTok(p, 1), "Expected normal or high for prio()")
-
-    p[0] = {'prio': kinds[p[1]]}
-
-
-def p_SendQualifier(p):
-    """SendQualifier : NESTED '(' Nested ')'
-                     | PRIO '(' Priority ')'"""
-    p[0] = p[3]
-
-
-def p_SendQualifierList(p):
-    """SendQualifierList : SendQualifier SendQualifierList
-                         | """
-    if len(p) > 1:
-        p[0] = p[1]
-        p[0].update(p[2])
+def p_SendSemantics(p):
+    """SendSemantics : ASYNC
+    | SYNC
+    | INTR"""
+    if p[1] == "async":
+        p[0] = ASYNC
+    elif p[1] == "sync":
+        p[0] = SYNC
     else:
-        p[0] = {}
-
-
-def p_SendSemanticsQual(p):
-    """SendSemanticsQual : SendQualifierList ASYNC
-                         | SendQualifierList SYNC
-                         | INTR"""
-    quals = {}
-    if len(p) == 3:
-        quals = p[1]
-        mtype = p[2]
-    else:
-        mtype = 'intr'
-
-    if mtype == 'async':
-        mtype = ASYNC
-    elif mtype == 'sync':
-        mtype = SYNC
-    elif mtype == 'intr':
-        mtype = INTR
-    else:
-        assert 0
-
-    p[0] = [quals.get('nested', NOT_NESTED), quals.get('prio', NORMAL_PRIORITY), mtype]
+        assert p[1] == "intr"
+        p[0] = INTR
 
 
 def p_OptionalProtocolSendSemanticsQual(p):
     """OptionalProtocolSendSemanticsQual : ProtocolSendSemanticsQual
-                                         | """
+    |"""
     if 2 == len(p):
         p[0] = p[1]
     else:
@@ -618,22 +596,22 @@ def p_OptionalProtocolSendSemanticsQual(p):
 
 def p_ProtocolSendSemanticsQual(p):
     """ProtocolSendSemanticsQual : ASYNC
-                                 | SYNC
-                                 | NESTED '(' UPTO Nested ')' ASYNC
-                                 | NESTED '(' UPTO Nested ')' SYNC
-                                 | INTR"""
-    if p[1] == 'nested':
+    | SYNC
+    | NESTED '(' UPTO Nested ')' ASYNC
+    | NESTED '(' UPTO Nested ')' SYNC
+    | INTR"""
+    if p[1] == "nested":
         mtype = p[6]
         nested = p[4]
     else:
         mtype = p[1]
         nested = NOT_NESTED
 
-    if mtype == 'async':
+    if mtype == "async":
         mtype = ASYNC
-    elif mtype == 'sync':
+    elif mtype == "sync":
         mtype = SYNC
-    elif mtype == 'intr':
+    elif mtype == "intr":
         mtype = INTR
     else:
         assert 0
@@ -643,8 +621,8 @@ def p_ProtocolSendSemanticsQual(p):
 
 def p_ParamList(p):
     """ParamList : ParamList ',' Param
-                 | Param
-                 | """
+    | Param
+    |"""
     if 1 == len(p):
         p[0] = []
     elif 2 == len(p):
@@ -655,8 +633,8 @@ def p_ParamList(p):
 
 
 def p_Param(p):
-    """Param : Type ID"""
-    p[0] = Param(locFromTok(p, 1), p[1], p[2])
+    """Param : Attributes Type ID"""
+    p[0] = Param(locFromTok(p, 2), p[2], p[3], p[1])
 
 
 def p_Type(p):
@@ -668,9 +646,9 @@ def p_Type(p):
 
 def p_BasicType(p):
     """BasicType : CxxID
-                 | CxxID '[' ']'
-                 | CxxID '?'
-                 | CxxUniquePtrInst"""
+    | CxxID '[' ']'
+    | CxxID '?'
+    | CxxUniquePtrInst"""
     # ID == CxxType; we forbid qnames here,
     # in favor of the |using| declaration
     if not isinstance(p[1], TypeSpec):
@@ -690,8 +668,9 @@ def p_BasicType(p):
 
 def p_MaybeNullable(p):
     """MaybeNullable : NULLABLE
-                     | """
-    p[0] = (2 == len(p))
+    |"""
+    p[0] = 2 == len(p)
+
 
 # --------------------
 # C++ stuff
@@ -699,7 +678,7 @@ def p_MaybeNullable(p):
 
 def p_CxxType(p):
     """CxxType : QualifiedID
-               | CxxID"""
+    | CxxID"""
     if isinstance(p[1], QualifiedId):
         p[0] = TypeSpec(p[1].loc, p[1])
     else:
@@ -709,7 +688,7 @@ def p_CxxType(p):
 
 def p_QualifiedID(p):
     """QualifiedID : QualifiedID COLONCOLON CxxID
-                   | CxxID COLONCOLON CxxID"""
+    | CxxID COLONCOLON CxxID"""
     if isinstance(p[1], QualifiedId):
         loc, id = p[3]
         p[1].qualify(id)
@@ -722,7 +701,7 @@ def p_QualifiedID(p):
 
 def p_CxxID(p):
     """CxxID : ID
-             | CxxTemplateInst"""
+    | CxxTemplateInst"""
     if isinstance(p[1], tuple):
         p[0] = p[1]
     else:
@@ -731,7 +710,7 @@ def p_CxxID(p):
 
 def p_CxxTemplateInst(p):
     """CxxTemplateInst : ID '<' ID '>'"""
-    p[0] = (locFromTok(p, 1), str(p[1]) + '<' + str(p[3]) + '>')
+    p[0] = (locFromTok(p, 1), str(p[1]) + "<" + str(p[3]) + ">")
 
 
 def p_CxxUniquePtrInst(p):
@@ -741,5 +720,4 @@ def p_CxxUniquePtrInst(p):
 
 def p_error(t):
     lineno, value = _safeLinenoValue(t)
-    _error(Loc(Parser.current.filename, lineno),
-           "bad syntax near `%s'", value)
+    _error(Loc(Parser.current.filename, lineno), "bad syntax near `%s'", value)
